# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy
import json
import os

from keystoneauth1 import session
import mock
import requests
from testtools import matchers

from ecl import exceptions
from ecl import format
from ecl import resource
from ecl.tests.unit import base
from ecl import utils


fake_parent = 'robert'
fake_name = 'rey'
fake_id = 99
fake_attr1 = 'lana'
fake_attr2 = 'del'

fake_resource = 'fake'
fake_resources = 'fakes'
fake_arguments = {'parent_name': fake_parent}
fake_base_path = '/fakes/%(parent_name)s/data'
fake_path = '/fakes/rey/data'

fake_data = {'id': fake_id,
             'enabled': True,
             'name': fake_name,
             'parent': fake_parent,
             'attr1': fake_attr1,
             'attr2': fake_attr2,
             'status': None}
fake_body = {fake_resource: fake_data}


class FakeParent(resource.Resource):
    id_attribute = "name"
    name = resource.prop('name')


class FakeResource(resource.Resource):

    resource_key = fake_resource
    resources_key = fake_resources
    base_path = fake_base_path

    allow_create = allow_retrieve = allow_update = True
    allow_delete = allow_list = allow_head = True

    enabled = resource.prop('enabled', type=format.BoolStr)
    name = resource.prop('name')
    parent = resource.prop('parent_name')
    first = resource.prop('attr1')
    second = resource.prop('attr2')
    third = resource.prop('attr3', alias='attr_three')
    status = resource.prop('status')


class FakeResourceNoKeys(FakeResource):

    resource_key = None
    resources_key = None


class PropTests(base.TestCase):

    def test_with_alias_and_type(self):
        class Test(resource.Resource):
            attr = resource.prop("attr1", alias="attr2", type=bool)

        t = Test(attrs={"attr2": 500})

        # Don't test with assertTrue because 500 evaluates to True.
        # Need to test that bool(500) happened and attr2 *is* True.
        self.assertIs(t.attr, True)

    def test_defaults(self):
        new_default = "new_default"

        class Test(resource.Resource):
            attr1 = resource.prop("attr1")
            attr2 = resource.prop("attr2", default=new_default)

        t = Test()

        self.assertIsNone(t.attr1)
        self.assertEqual(new_default, t.attr2)

        # When the default value is passed in, it is left untouched.
        # Check that attr2 is literally the same object we set as default.
        t.attr2 = new_default
        self.assertIs(new_default, t.attr2)

        not_default = 'not default'
        t2 = Test({'attr2': not_default})
        self.assertEqual(not_default, t2.attr2)

        # Assert that if the default is passed in, it overrides the previously
        # set value (bug #1425996)
        t2.attr2 = new_default
        self.assertEqual(new_default, t2.attr2)

    def test_get_without_instance(self):
        self.assertIsNone(FakeResource.name)

    def test_set_ValueError(self):
        class Test(resource.Resource):
            attr = resource.prop("attr", type=int)

        t = Test()

        def should_raise():
            t.attr = "this is not an int"

        self.assertThat(should_raise, matchers.raises(ValueError))

    def test_set_TypeError(self):
        class Type(object):
            def __init__(self):
                pass

        class Test(resource.Resource):
            attr = resource.prop("attr", type=Type)

        t = Test()

        def should_raise():
            t.attr = "this type takes no args"

        self.assertThat(should_raise, matchers.raises(TypeError))

    def test_resource_type(self):
        class FakestResource(resource.Resource):
            shortstop = resource.prop("shortstop", type=FakeResource)
            third_base = resource.prop("third_base", type=FakeResource)

        sot = FakestResource()
        id1 = "Ernie Banks"
        id2 = "Ron Santo"
        sot.shortstop = id1
        sot.third_base = id2

        resource1 = FakeResource.new(id=id1)
        self.assertEqual(resource1, sot.shortstop)
        self.assertEqual(id1, sot.shortstop.id)
        self.assertEqual(FakeResource, type(sot.shortstop))

        resource2 = FakeResource.new(id=id2)
        self.assertEqual(resource2, sot.third_base)
        self.assertEqual(id2, sot.third_base.id)
        self.assertEqual(FakeResource, type(sot.third_base))

        sot2 = FakestResource()
        sot2.shortstop = resource1
        sot2.third_base = resource2
        self.assertEqual(resource1, sot2.shortstop)
        self.assertEqual(id1, sot2.shortstop.id)
        self.assertEqual(FakeResource, type(sot2.shortstop))
        self.assertEqual(resource2, sot2.third_base)
        self.assertEqual(id2, sot2.third_base.id)
        self.assertEqual(FakeResource, type(sot2.third_base))

        body = {
            "shortstop": id1,
            "third_base": id2
        }
        sot3 = FakestResource(body)
        self.assertEqual(FakeResource({"id": id1}), sot3.shortstop)
        self.assertEqual(FakeResource({"id": id2}), sot3.third_base)

    def test_set_alias_same_name(self):
        class Test(resource.Resource):
            attr = resource.prop("something", alias="attr")

        val = "hey"
        args = {"something": val}
        sot = Test(args)

        self.assertEqual(val, sot._attrs["something"])
        self.assertEqual(val, sot.attr)

    def test_property_is_none(self):
        class Test(resource.Resource):
            attr = resource.prop("something", type=dict)

        args = {"something": None}
        sot = Test(args)

        self.assertIsNone(sot._attrs["something"])
        self.assertIsNone(sot.attr)


class HeaderTests(base.TestCase):
    class Test(resource.Resource):
        base_path = "/ramones"
        service = "punk"
        allow_create = True
        allow_update = True
        hey = resource.header("vocals")
        ho = resource.header("guitar")
        letsgo = resource.header("bass")

    def test_get(self):
        val = "joey"
        args = {"vocals": val}
        sot = HeaderTests.Test({'headers': args})
        self.assertEqual(val, sot.hey)
        self.assertIsNone(sot.ho)
        self.assertIsNone(sot.letsgo)

    def test_set_new(self):
        args = {"vocals": "joey", "bass": "deedee"}
        sot = HeaderTests.Test({'headers': args})
        sot._reset_dirty()
        sot.ho = "johnny"
        self.assertEqual("johnny", sot.ho)
        self.assertTrue(sot.is_dirty)

    def test_set_old(self):
        args = {"vocals": "joey", "bass": "deedee"}
        sot = HeaderTests.Test({'headers': args})
        sot._reset_dirty()
        sot.letsgo = "cj"
        self.assertEqual("cj", sot.letsgo)
        self.assertTrue(sot.is_dirty)

    def test_set_brand_new(self):
        sot = HeaderTests.Test({'headers': {}})
        sot._reset_dirty()
        sot.ho = "johnny"
        self.assertEqual("johnny", sot.ho)
        self.assertTrue(sot.is_dirty)
        self.assertEqual({'headers': {"guitar": "johnny"}}, sot)

    def test_1428342(self):
        sot = HeaderTests.Test({'headers':
                               requests.structures.CaseInsensitiveDict()})

        self.assertIsNone(sot.hey)

    def test_create_update_headers(self):
        sot = HeaderTests.Test()
        sot._reset_dirty()
        sot.ho = "johnny"
        sot.letsgo = "deedee"
        response = mock.Mock()
        response_body = {'id': 1}
        response.json = mock.Mock(return_value=response_body)
        response.headers = None
        sess = mock.Mock()
        sess.post = mock.Mock(return_value=response)
        sess.put = mock.Mock(return_value=response)

        sot.create(sess)
        headers = {'guitar': 'johnny', 'bass': 'deedee'}
        sess.post.assert_called_with(HeaderTests.Test.base_path,
                                     endpoint_filter=HeaderTests.Test.service,
                                     headers=headers,
                                     json={})

        sot['id'] = 1
        sot.letsgo = "cj"
        headers = {'guitar': 'johnny', 'bass': 'cj'}
        sot.update(sess)
        sess.put.assert_called_with('ramones/1',
                                    endpoint_filter=HeaderTests.Test.service,
                                    headers=headers,
                                    json={})


class ResourceTests(base.TestCase):

    def setUp(self):
        super(ResourceTests, self).setUp()
        self.session = mock.Mock(spec=session.Session)
        self.session.get_filter = mock.Mock(return_value={})

    def assertCalledURL(self, method, url):
        # call_args gives a tuple of *args and tuple of **kwargs.
        # Check that the first arg in *args (the URL) has our url.
        self.assertEqual(method.call_args[0][0], url)

    def test_empty_id(self):
        resp = mock.Mock()
        resp.json = mock.Mock(return_value=fake_body)
        self.session.get.return_value = resp

        obj = FakeResource.new(**fake_arguments)
        self.assertEqual(obj, obj.get(self.session))

        self.assertEqual(fake_id, obj.id)
        self.assertEqual(fake_name, obj['name'])
        self.assertEqual(fake_attr1, obj['attr1'])
        self.assertEqual(fake_attr2, obj['attr2'])

        self.assertEqual(fake_name, obj.name)
        self.assertEqual(fake_attr1, obj.first)
        self.assertEqual(fake_attr2, obj.second)

    def test_not_allowed(self):
        class Nope(resource.Resource):
            allow_create = allow_retrieve = allow_update = False
            allow_delete = allow_list = allow_head = False

        nope = Nope()

        def cant_create():
            nope.create_by_id(1, 2)

        def cant_retrieve():
            nope.get_data_by_id(1, 2)

        def cant_update():
            nope.update_by_id(1, 2, 3)

        def cant_delete():
            nope.delete_by_id(1, 2)

        def cant_list():
            for i in nope.list(1):
                pass

        def cant_head():
            nope.head_data_by_id(1, 2)

        self.assertThat(cant_create,
                        matchers.raises(exceptions.MethodNotSupported))
        self.assertThat(cant_retrieve,
                        matchers.raises(exceptions.MethodNotSupported))
        self.assertThat(cant_update,
                        matchers.raises(exceptions.MethodNotSupported))
        self.assertThat(cant_delete,
                        matchers.raises(exceptions.MethodNotSupported))
        self.assertThat(cant_list,
                        matchers.raises(exceptions.MethodNotSupported))
        self.assertThat(cant_head,
                        matchers.raises(exceptions.MethodNotSupported))

    def _test_create_by_id(self, key, response_value, response_body,
                           attrs, json_body, response_headers=None):

        class FakeResource2(FakeResource):
            resource_key = key
            service = "my_service"

        response = mock.Mock()
        response.json = mock.Mock(return_value=response_body)
        response.headers = response_headers
        expected_resp = response_value.copy()
        if response_headers:
            expected_resp.update({'headers': response_headers})

        sess = mock.Mock()
        sess.put = mock.Mock(return_value=response)
        sess.post = mock.Mock(return_value=response)

        resp = FakeResource2.create_by_id(sess, attrs)
        self.assertEqual(expected_resp, resp)
        sess.post.assert_called_with(FakeResource2.base_path,
                                     endpoint_filter=FakeResource2.service,
                                     json=json_body)

        r_id = "my_id"
        resp = FakeResource2.create_by_id(sess, attrs, resource_id=r_id)
        self.assertEqual(response_value, resp)
        sess.put.assert_called_with(
            utils.urljoin(FakeResource2.base_path, r_id),
            endpoint_filter=FakeResource2.service,
            json=json_body)

        path_args = {"parent_name": "my_name"}
        resp = FakeResource2.create_by_id(sess, attrs, path_args=path_args)
        self.assertEqual(response_value, resp)
        sess.post.assert_called_with(FakeResource2.base_path % path_args,
                                     endpoint_filter=FakeResource2.service,
                                     json=json_body)

        resp = FakeResource2.create_by_id(sess, attrs, resource_id=r_id,
                                          path_args=path_args)
        self.assertEqual(response_value, resp)
        sess.put.assert_called_with(
            utils.urljoin(FakeResource2.base_path % path_args, r_id),
            endpoint_filter=FakeResource2.service,
            json=json_body)

    def test_create_without_resource_key(self):
        key = None
        response_value = {"a": 1, "b": 2, "c": 3}
        response_body = response_value
        attrs = response_value
        json_body = attrs
        self._test_create_by_id(key, response_value, response_body,
                                attrs, json_body)

    def test_create_with_response_headers(self):
        key = None
        response_value = {"a": 1, "b": 2, "c": 3}
        response_body = response_value
        response_headers = {'location': 'foo'}
        attrs = response_value.copy()
        json_body = attrs
        self._test_create_by_id(key, response_value, response_body,
                                attrs, json_body,
                                response_headers=response_headers)

    def test_create_with_resource_key(self):
        key = "my_key"
        response_value = {"a": 1, "b": 2, "c": 3}
        response_body = {key: response_value}
        attrs = response_body
        json_body = {key: attrs}
        self._test_create_by_id(key, response_value, response_body,
                                attrs, json_body)

    def _test_get_data_by_id(self, key, response_value, response_body):
        class FakeResource2(FakeResource):
            resource_key = key
            service = "my_service"

        response = mock.Mock()
        response.json = mock.Mock(return_value=response_body)

        sess = mock.Mock()
        sess.get = mock.Mock(return_value=response)

        r_id = "my_id"
        resp = FakeResource2.get_data_by_id(sess, resource_id=r_id)
        self.assertEqual(response_value, resp)
        sess.get.assert_called_with(
            utils.urljoin(FakeResource2.base_path, r_id),
            endpoint_filter=FakeResource2.service)

        path_args = {"parent_name": "my_name"}
        resp = FakeResource2.get_data_by_id(sess, resource_id=r_id,
                                            path_args=path_args)
        self.assertEqual(response_value, resp)
        sess.get.assert_called_with(
            utils.urljoin(FakeResource2.base_path % path_args, r_id),
            endpoint_filter=FakeResource2.service)

    def test_get_data_without_resource_key(self):
        key = None
        response_value = {"a": 1, "b": 2, "c": 3}
        response_body = response_value
        self._test_get_data_by_id(key, response_value, response_body)

    def test_get_data_with_resource_key(self):
        key = "my_key"
        response_value = {"a": 1, "b": 2, "c": 3}
        response_body = {key: response_value}
        self._test_get_data_by_id(key, response_value, response_body)

    def _test_head_data_by_id(self, key, response_value):
        class FakeResource2(FakeResource):
            resource_key = key
            service = "my_service"

        response = mock.Mock()
        response.headers = response_value

        sess = mock.Mock()
        sess.head = mock.Mock(return_value=response)

        r_id = "my_id"
        resp = FakeResource2.head_data_by_id(sess, resource_id=r_id)
        self.assertEqual({'headers': response_value}, resp)
        headers = {'Accept': ''}
        sess.head.assert_called_with(
            utils.urljoin(FakeResource2.base_path, r_id),
            endpoint_filter=FakeResource2.service,
            headers=headers)

        path_args = {"parent_name": "my_name"}
        resp = FakeResource2.head_data_by_id(sess, resource_id=r_id,
                                             path_args=path_args)
        self.assertEqual({'headers': response_value}, resp)
        headers = {'Accept': ''}
        sess.head.assert_called_with(
            utils.urljoin(FakeResource2.base_path % path_args, r_id),
            endpoint_filter=FakeResource2.service,
            headers=headers)

    def test_head_data_without_resource_key(self):
        key = None
        response_value = {"key1": "value1", "key2": "value2"}
        self._test_head_data_by_id(key, response_value)

    def test_head_data_with_resource_key(self):
        key = "my_key"
        response_value = {"key1": "value1", "key2": "value2"}
        self._test_head_data_by_id(key, response_value)

    def _test_update_by_id(self, key, response_value, response_body,
                           attrs, json_body, response_headers=None):

        class FakeResource2(FakeResource):
            patch_update = True
            resource_key = key
            service = "my_service"

        response = mock.Mock()
        response.json = mock.Mock(return_value=response_body)
        response.headers = response_headers
        expected_resp = response_value.copy()
        if response_headers:
            expected_resp.update({'headers': response_headers})

        sess = mock.Mock()
        sess.patch = mock.Mock(return_value=response)

        r_id = "my_id"
        resp = FakeResource2.update_by_id(sess, r_id, attrs)
        self.assertEqual(expected_resp, resp)
        sess.patch.assert_called_with(
            utils.urljoin(FakeResource2.base_path, r_id),
            endpoint_filter=FakeResource2.service,
            json=json_body)

        path_args = {"parent_name": "my_name"}
        resp = FakeResource2.update_by_id(sess, r_id, attrs,
                                          path_args=path_args)
        self.assertEqual(expected_resp, resp)
        sess.patch.assert_called_with(
            utils.urljoin(FakeResource2.base_path % path_args, r_id),
            endpoint_filter=FakeResource2.service,
            json=json_body)

    def test_update_without_resource_key(self):
        key = None
        response_value = {"a": 1, "b": 2, "c": 3}
        response_body = response_value
        attrs = response_value
        json_body = attrs
        self._test_update_by_id(key, response_value, response_body,
                                attrs, json_body)

    def test_update_with_resource_key(self):
        key = "my_key"
        response_value = {"a": 1, "b": 2, "c": 3}
        response_body = {key: response_value}
        attrs = response_value
        json_body = {key: attrs}
        self._test_update_by_id(key, response_value, response_body,
                                attrs, json_body)

    def test_update_with_response_headers(self):
        key = "my_key"
        response_value = {"a": 1, "b": 2, "c": 3}
        response_body = {key: response_value}
        response_headers = {'location': 'foo'}
        attrs = response_value.copy()
        json_body = {key: attrs}
        self._test_update_by_id(key, response_value, response_body,
                                attrs, json_body,
                                response_headers=response_headers)

    def test_delete_by_id(self):
        class FakeResource2(FakeResource):
            service = "my_service"

        sess = mock.Mock()
        sess.delete = mock.Mock(return_value=None)

        r_id = "my_id"
        resp = FakeResource2.delete_by_id(sess, r_id)
        self.assertIsNone(resp)
        headers = {'Accept': ''}
        sess.delete.assert_called_with(
            utils.urljoin(FakeResource2.base_path, r_id),
            endpoint_filter=FakeResource2.service,
            headers=headers)

        path_args = {"parent_name": "my_name"}
        resp = FakeResource2.delete_by_id(sess, r_id, path_args=path_args)
        self.assertIsNone(resp)
        headers = {'Accept': ''}
        sess.delete.assert_called_with(
            utils.urljoin(FakeResource2.base_path % path_args, r_id),
            endpoint_filter=FakeResource2.service,
            headers=headers)

    def test_create(self):
        resp = mock.Mock()
        resp.json = mock.Mock(return_value=fake_body)
        resp.headers = {'location': 'foo'}
        self.session.post = mock.Mock(return_value=resp)

        # Create resource with subset of attributes in order to
        # verify create refreshes all attributes from response.
        obj = FakeResource.new(parent_name=fake_parent,
                               name=fake_name,
                               enabled=True,
                               attr1=fake_attr1)

        self.assertEqual(obj, obj.create(self.session))
        self.assertFalse(obj.is_dirty)

        last_req = self.session.post.call_args[1]["json"][
            FakeResource.resource_key]

        self.assertEqual(4, len(last_req))
        self.assertTrue(last_req['enabled'])
        self.assertEqual(fake_parent, last_req['parent_name'])
        self.assertEqual(fake_name, last_req['name'])
        self.assertEqual(fake_attr1, last_req['attr1'])

        self.assertTrue(obj['enabled'])
        self.assertEqual(fake_name, obj['name'])
        self.assertEqual(fake_parent, obj['parent_name'])
        self.assertEqual(fake_attr1, obj['attr1'])
        self.assertEqual(fake_attr2, obj['attr2'])
        self.assertIsNone(obj['status'])

        self.assertTrue(obj.enabled)
        self.assertEqual(fake_id, obj.id)
        self.assertEqual(fake_name, obj.name)
        self.assertEqual(fake_parent, obj.parent_name)
        self.assertEqual(fake_parent, obj.parent)
        self.assertEqual(fake_attr1, obj.first)
        self.assertEqual(fake_attr1, obj.attr1)
        self.assertEqual(fake_attr2, obj.second)
        self.assertEqual(fake_attr2, obj.attr2)
        self.assertIsNone(obj.status)
        self.assertEqual('foo', obj.location)

    def test_get(self):
        resp = mock.Mock()
        resp.json = mock.Mock(return_value=fake_body)
        resp.headers = {'location': 'foo'}
        self.session.get = mock.Mock(return_value=resp)

        # Create resource with subset of attributes in order to
        # verify get refreshes all attributes from response.
        obj = FakeResource.from_id(str(fake_id))
        obj['parent_name'] = fake_parent

        self.assertEqual(obj, obj.get(self.session))

        # Check that the proper URL is being built.
        self.assertCalledURL(self.session.get,
                             os.path.join(fake_base_path % fake_arguments,
                                          str(fake_id))[1:])

        self.assertTrue(obj['enabled'])
        self.assertEqual(fake_name, obj['name'])
        self.assertEqual(fake_parent, obj['parent_name'])
        self.assertEqual(fake_attr1, obj['attr1'])
        self.assertEqual(fake_attr2, obj['attr2'])
        self.assertIsNone(obj['status'])

        self.assertTrue(obj.enabled)
        self.assertEqual(fake_id, obj.id)
        self.assertEqual(fake_name, obj.name)
        self.assertEqual(fake_parent, obj.parent_name)
        self.assertEqual(fake_parent, obj.parent)
        self.assertEqual(fake_attr1, obj.first)
        self.assertEqual(fake_attr1, obj.attr1)
        self.assertEqual(fake_attr2, obj.second)
        self.assertEqual(fake_attr2, obj.attr2)
        self.assertIsNone(obj.status)
        self.assertIsNone(obj.location)

    def test_get_by_id(self):
        resp = mock.Mock()
        resp.json = mock.Mock(return_value=fake_body)
        self.session.get = mock.Mock(return_value=resp)

        obj = FakeResource.get_by_id(self.session, fake_id,
                                     path_args=fake_arguments)

        # Check that the proper URL is being built.
        self.assertCalledURL(self.session.get,
                             os.path.join(fake_base_path % fake_arguments,
                                          str(fake_id))[1:])

        self.assertEqual(fake_id, obj.id)
        self.assertEqual(fake_name, obj['name'])
        self.assertEqual(fake_attr1, obj['attr1'])
        self.assertEqual(fake_attr2, obj['attr2'])

        self.assertEqual(fake_name, obj.name)
        self.assertEqual(fake_attr1, obj.first)
        self.assertEqual(fake_attr2, obj.second)

    def test_get_by_id_with_headers(self):
        header1 = "fake-value1"
        header2 = "fake-value2"
        headers = {"header1": header1,
                   "header2": header2}

        resp = mock.Mock(headers=headers)
        resp.json = mock.Mock(return_value=fake_body)
        self.session.get = mock.Mock(return_value=resp)

        class FakeResource2(FakeResource):
            header1 = resource.header("header1")
            header2 = resource.header("header2")

        obj = FakeResource2.get_by_id(self.session, fake_id,
                                      path_args=fake_arguments,
                                      include_headers=True)

        self.assertCalledURL(self.session.get,
                             os.path.join(fake_base_path % fake_arguments,
                                          str(fake_id))[1:])

        self.assertEqual(fake_id, obj.id)
        self.assertEqual(fake_name, obj['name'])
        self.assertEqual(fake_attr1, obj['attr1'])
        self.assertEqual(fake_attr2, obj['attr2'])
        self.assertEqual(header1, obj['headers']['header1'])
        self.assertEqual(header2, obj['headers']['header2'])

        self.assertEqual(fake_name, obj.name)
        self.assertEqual(fake_attr1, obj.first)
        self.assertEqual(fake_attr2, obj.second)
        self.assertEqual(header1, obj.header1)
        self.assertEqual(header2, obj.header2)

    def test_head_by_id(self):
        class FakeResource2(FakeResource):
            header1 = resource.header("header1")
            header2 = resource.header("header2")

        resp = mock.Mock(headers={"header1": "one", "header2": "two"})
        self.session.head = mock.Mock(return_value=resp)

        obj = FakeResource2.head_by_id(self.session, fake_id,
                                       path_args=fake_arguments)

        self.assertCalledURL(self.session.head,
                             os.path.join(fake_base_path % fake_arguments,
                                          str(fake_id))[1:])

        self.assertEqual('one', obj['headers']['header1'])
        self.assertEqual('two', obj['headers']['header2'])

        self.assertEqual('one', obj.header1)
        self.assertEqual('two', obj.header2)

    def test_patch_update(self):
        class FakeResourcePatch(FakeResource):
            patch_update = True

        resp = mock.Mock()
        resp.json = mock.Mock(return_value=fake_body)
        resp.headers = {'location': 'foo'}
        self.session.patch = mock.Mock(return_value=resp)

        # Create resource with subset of attributes in order to
        # verify update refreshes all attributes from response.
        obj = FakeResourcePatch.new(id=fake_id, parent_name=fake_parent,
                                    name=fake_name, attr1=fake_attr1)
        self.assertTrue(obj.is_dirty)

        self.assertEqual(obj, obj.update(self.session))
        self.assertFalse(obj.is_dirty)

        self.assertCalledURL(self.session.patch,
                             os.path.join(fake_base_path % fake_arguments,
                                          str(fake_id))[1:])

        last_req = self.session.patch.call_args[1]["json"][
            FakeResource.resource_key]

        self.assertEqual(3, len(last_req))
        self.assertEqual(fake_parent, last_req['parent_name'])
        self.assertEqual(fake_name, last_req['name'])
        self.assertEqual(fake_attr1, last_req['attr1'])

        self.assertTrue(obj['enabled'])
        self.assertEqual(fake_name, obj['name'])
        self.assertEqual(fake_parent, obj['parent_name'])
        self.assertEqual(fake_attr1, obj['attr1'])
        self.assertEqual(fake_attr2, obj['attr2'])
        self.assertIsNone(obj['status'])

        self.assertTrue(obj.enabled)
        self.assertEqual(fake_id, obj.id)
        self.assertEqual(fake_name, obj.name)
        self.assertEqual(fake_parent, obj.parent_name)
        self.assertEqual(fake_parent, obj.parent)
        self.assertEqual(fake_attr1, obj.first)
        self.assertEqual(fake_attr1, obj.attr1)
        self.assertEqual(fake_attr2, obj.second)
        self.assertEqual(fake_attr2, obj.attr2)
        self.assertIsNone(obj.status)
        self.assertEqual('foo', obj.location)

    def test_put_update(self):
        class FakeResourcePut(FakeResource):
            # This is False by default, but explicit for this test.
            patch_update = False

        resp = mock.Mock()
        resp.json = mock.Mock(return_value=fake_body)
        resp.headers = {'location': 'foo'}
        self.session.put = mock.Mock(return_value=resp)

        # Create resource with subset of attributes in order to
        # verify update refreshes all attributes from response.
        obj = FakeResourcePut.new(id=fake_id, parent_name=fake_parent,
                                  name=fake_name, attr1=fake_attr1)
        self.assertTrue(obj.is_dirty)

        self.assertEqual(obj, obj.update(self.session))
        self.assertFalse(obj.is_dirty)

        self.assertCalledURL(self.session.put,
                             os.path.join(fake_base_path % fake_arguments,
                                          str(fake_id))[1:])

        last_req = self.session.put.call_args[1]["json"][
            FakeResource.resource_key]

        self.assertEqual(3, len(last_req))
        self.assertEqual(fake_parent, last_req['parent_name'])
        self.assertEqual(fake_name, last_req['name'])
        self.assertEqual(fake_attr1, last_req['attr1'])

        self.assertTrue(obj['enabled'])
        self.assertEqual(fake_name, obj['name'])
        self.assertEqual(fake_parent, obj['parent_name'])
        self.assertEqual(fake_attr1, obj['attr1'])
        self.assertEqual(fake_attr2, obj['attr2'])
        self.assertIsNone(obj['status'])

        self.assertTrue(obj.enabled)
        self.assertEqual(fake_id, obj.id)
        self.assertEqual(fake_name, obj.name)
        self.assertEqual(fake_parent, obj.parent_name)
        self.assertEqual(fake_parent, obj.parent)
        self.assertEqual(fake_attr1, obj.first)
        self.assertEqual(fake_attr1, obj.attr1)
        self.assertEqual(fake_attr2, obj.second)
        self.assertEqual(fake_attr2, obj.attr2)
        self.assertIsNone(obj.status)
        self.assertEqual('foo', obj.location)

    def test_update_early_exit(self):
        obj = FakeResource()
        obj._dirty = []  # Bail out early if there's nothing to update.

        self.assertIsNone(obj.update("session"))

    def test_update_no_id_attribute(self):
        obj = FakeResource.existing(id=1, attr="value1",
                                    parent_name=fake_parent)
        obj.first = "value2"  # Make it dirty
        obj.update_by_id = mock.Mock(return_value=dict())
        # If no id_attribute is returned in the update response, make sure
        # we handle the resulting KeyError.
        self.assertEqual(obj, obj.update("session"))

    def test_delete(self):
        obj = FakeResource({"id": fake_id, "parent_name": fake_parent})
        obj.delete(self.session)

        self.assertCalledURL(self.session.delete,
                             os.path.join(fake_base_path % fake_arguments,
                                          str(fake_id))[1:])

    def _test_list(self, resource_class):
        results = [fake_data.copy(), fake_data.copy(), fake_data.copy()]
        for i in range(len(results)):
            results[i]['id'] = fake_id + i
        if resource_class.resources_key is not None:
            body = {resource_class.resources_key:
                    self._get_expected_results()}
            sentinel = {resource_class.resources_key: []}
        else:
            body = self._get_expected_results()
            sentinel = []
        resp1 = mock.Mock()
        resp1.json = mock.Mock(return_value=body)
        resp2 = mock.Mock()
        resp2.json = mock.Mock(return_value=sentinel)
        self.session.get.side_effect = [resp1, resp2]

        objs = list(resource_class.list(self.session, path_args=fake_arguments,
                                        paginated=True))

        params = {'limit': 3, 'marker': results[-1]['id']}
        self.assertEqual(params, self.session.get.call_args[1]['params'])
        self.assertEqual(3, len(objs))
        for obj in objs:
            self.assertIn(obj.id, range(fake_id, fake_id + 3))
            self.assertEqual(fake_name, obj['name'])
            self.assertEqual(fake_name, obj.name)
            self.assertIsInstance(obj, FakeResource)

    def _get_expected_results(self):
        results = [fake_data.copy(), fake_data.copy(), fake_data.copy()]
        for i in range(len(results)):
            results[i]['id'] = fake_id + i
        return results

    def test_list_keyed_resource(self):
        self._test_list(FakeResource)

    def test_list_non_keyed_resource(self):
        self._test_list(FakeResourceNoKeys)

    def _test_list_call_count(self, paginated):
        # Test that we've only made one call to receive all data
        results = [fake_data.copy(), fake_data.copy(), fake_data.copy()]
        resp = mock.Mock()
        resp.json = mock.Mock(return_value={fake_resources: results})
        attrs = {"get.return_value": resp}
        session = mock.Mock(**attrs)

        list(FakeResource.list(session, params={'limit': len(results) + 1},
                               path_args=fake_arguments,
                               paginated=paginated))

        # Ensure we only made one call to complete this.
        self.assertEqual(1, session.get.call_count)

    def test_list_bail_out(self):
        # When we get less data than limit, make sure we made one call
        self._test_list_call_count(True)

    def test_list_nonpaginated(self):
        # When we call with paginated=False, make sure we made one call
        self._test_list_call_count(False)

    def test_determine_limit(self):
        full_page = [fake_data.copy(), fake_data.copy(), fake_data.copy()]
        last_page = [fake_data.copy()]

        session = mock.Mock()
        session.get = mock.Mock()
        full_response = mock.Mock()
        response_body = {FakeResource.resources_key: full_page}
        full_response.json = mock.Mock(return_value=response_body)
        last_response = mock.Mock()
        response_body = {FakeResource.resources_key: last_page}
        last_response.json = mock.Mock(return_value=response_body)
        pages = [full_response, full_response, last_response]
        session.get.side_effect = pages

        # Don't specify a limit. Resource.list will determine the limit
        # is 3 based on the first `full_page`.
        results = list(FakeResource.list(session, path_args=fake_arguments,
                       paginated=True))

        self.assertEqual(session.get.call_count, len(pages))
        self.assertEqual(len(full_page + full_page + last_page), len(results))

    def test_empty_list(self):
        page = []

        session = mock.Mock()
        session.get = mock.Mock()
        full_response = mock.Mock()
        response_body = {FakeResource.resources_key: page}
        full_response.json = mock.Mock(return_value=response_body)
        pages = [full_response]
        session.get.side_effect = pages

        results = list(FakeResource.list(session, path_args=fake_arguments,
                       paginated=True))

        self.assertEqual(session.get.call_count, len(pages))
        self.assertEqual(len(page), len(results))

    def test_attrs_name(self):
        obj = FakeResource()

        self.assertIsNone(obj.name)
        del obj.name

    def test_to_dict(self):
        kwargs = {
            'enabled': True,
            'name': 'FOO',
            'parent': 'dad',
            'attr1': 'BAR',
            'attr2': ['ZOO', 'BAZ'],
            'status': 'Active',
            'headers': {
                'key': 'value'
            }
        }
        obj = FakeResource(kwargs)
        res = obj.to_dict()
        self.assertIsInstance(res, dict)
        self.assertTrue(res['enabled'])
        self.assertEqual('FOO', res['name'])
        self.assertEqual('dad', res['parent'])
        self.assertEqual('BAR', res['attr1'])
        self.assertEqual(['ZOO', 'BAZ'], res['attr2'])
        self.assertEqual('Active', res['status'])
        self.assertNotIn('headers', res)

    def test_composite_attr_happy(self):
        obj = FakeResource.existing(**{'attr3': '3'})

        try:
            self.assertEqual('3', obj.third)
        except AttributeError:
            self.fail("third was not found as expected")

    def test_composite_attr_fallback(self):
        obj = FakeResource.existing(**{'attr_three': '3'})

        try:
            self.assertEqual('3', obj.third)
        except AttributeError:
            self.fail("third was not found in fallback as expected")

    def test_id_del(self):

        class Test(resource.Resource):
            id_attribute = "my_id"

        attrs = {"my_id": 100}
        t = Test(attrs=attrs)

        self.assertEqual(attrs["my_id"], t.id)
        del t.id
        self.assertTrue(Test.id_attribute not in t._attrs)

    def test_from_name_with_name(self):
        name = "Ernie Banks"

        obj = FakeResource.from_name(name)
        self.assertEqual(name, obj.name)

    def test_from_id_with_name(self):
        name = "Sandy Koufax"

        obj = FakeResource.from_id(name)
        self.assertEqual(name, obj.id)

    def test_from_id_with_object(self):
        name = "Mickey Mantle"
        obj = FakeResource.new(name=name)

        new_obj = FakeResource.from_id(obj)
        self.assertIs(new_obj, obj)
        self.assertEqual(obj.name, new_obj.name)

    def test_from_id_with_bad_value(self):
        def should_raise():
            FakeResource.from_id(3.14)

        self.assertThat(should_raise, matchers.raises(ValueError))

    def test_dirty_list(self):
        class Test(resource.Resource):
            attr = resource.prop("attr")

        # Check if dirty after setting by prop
        sot1 = Test()
        self.assertFalse(sot1.is_dirty)
        sot1.attr = 1
        self.assertTrue(sot1.is_dirty)

        # Check if dirty after setting by mapping
        sot2 = Test()
        sot2["attr"] = 1
        self.assertTrue(sot1.is_dirty)

        # Check if dirty after creation
        sot3 = Test({"attr": 1})
        self.assertTrue(sot3.is_dirty)

    def test_update_attrs(self):
        class Test(resource.Resource):
            moe = resource.prop("the-attr")
            larry = resource.prop("the-attr2")
            curly = resource.prop("the-attr3", type=int)
            shemp = resource.prop("the-attr4")

        value1 = "one"
        value2 = "two"
        value3 = "3"
        value4 = "fore"
        value5 = "fiver"

        sot = Test({"the-attr": value1})

        sot.update_attrs({"the-attr2": value2, "notprop": value4})
        self.assertTrue(sot.is_dirty)
        self.assertEqual(value1, sot.moe)
        self.assertEqual(value1, sot["the-attr"])
        self.assertEqual(value2, sot.larry)
        self.assertEqual(value4, sot.notprop)

        sot._reset_dirty()

        sot.update_attrs(curly=value3)
        self.assertTrue(sot.is_dirty)
        self.assertEqual(int, type(sot.curly))
        self.assertEqual(int(value3), sot.curly)

        sot._reset_dirty()

        sot.update_attrs(**{"the-attr4": value5})
        self.assertTrue(sot.is_dirty)
        self.assertEqual(value5, sot.shemp)

    def test_get_id(self):
        class Test(resource.Resource):
            pass

        ID = "an id"
        res = Test({"id": ID})

        self.assertEqual(ID, resource.Resource.get_id(ID))
        self.assertEqual(ID, resource.Resource.get_id(res))

    def test_convert_ids(self):
        class TestResourceFoo(resource.Resource):
            pass

        class TestResourceBar(resource.Resource):
            pass

        resfoo = TestResourceFoo({'id': 'FAKEFOO'})
        resbar = TestResourceBar({'id': 'FAKEBAR'})

        self.assertIsNone(resource.Resource.convert_ids(None))
        attrs = {
            'key1': 'value1'
        }
        self.assertEqual(attrs, resource.Resource.convert_ids(attrs))

        attrs = {
            'foo': resfoo,
            'bar': resbar,
            'other': 'whatever',
        }
        res = resource.Resource.convert_ids(attrs)
        self.assertEqual('FAKEFOO', res['foo'])
        self.assertEqual('FAKEBAR', res['bar'])
        self.assertEqual('whatever', res['other'])

    def test_repr(self):
        fr = FakeResource()
        fr._loaded = False
        fr.first = "hey"
        fr.second = "hi"
        fr.third = "nah"
        the_repr = repr(fr)
        the_repr = the_repr.replace('ecl.tests.unit.test_resource.', '')
        result = eval(the_repr)
        self.assertEqual(fr._loaded, result._loaded)
        self.assertEqual(fr.first, result.first)
        self.assertEqual(fr.second, result.second)
        self.assertEqual(fr.third, result.third)

    def test_id_attribute(self):
        faker = FakeResource(fake_data)
        self.assertEqual(fake_id, faker.id)
        faker.id_attribute = 'name'
        self.assertEqual(fake_name, faker.id)
        faker.id_attribute = 'attr1'
        self.assertEqual(fake_attr1, faker.id)
        faker.id_attribute = 'attr2'
        self.assertEqual(fake_attr2, faker.id)
        faker.id_attribute = 'id'
        self.assertEqual(fake_id, faker.id)

    def test_name_attribute(self):
        class Person_ES(resource.Resource):
            name_attribute = "nombre"
            nombre = resource.prop('nombre')

        name = "Brian"
        args = {'nombre': name}

        person = Person_ES(args)
        self.assertEqual(name, person.nombre)
        self.assertEqual(name, person.name)

        new_name = "Julien"
        person.name = new_name
        self.assertEqual(new_name, person.nombre)
        self.assertEqual(new_name, person.name)

    def test_boolstr_prop(self):
        faker = FakeResource(fake_data)
        self.assertTrue(faker.enabled)
        self.assertTrue(faker['enabled'])

        faker._attrs['enabled'] = False
        self.assertFalse(faker.enabled)
        self.assertFalse(faker['enabled'])

        # should fail fast
        def set_invalid():
            faker.enabled = 'INVALID'
        self.assertRaises(ValueError, set_invalid)


class ResourceMapping(base.TestCase):

    def test__getitem(self):
        value = 10

        class Test(resource.Resource):
            attr = resource.prop("attr")

        t = Test(attrs={"attr": value})

        self.assertEqual(value, t["attr"])

    def test__setitem__existing_item_changed(self):

        class Test(resource.Resource):
            pass

        t = Test()
        key = "attr"
        value = 1
        t[key] = value

        self.assertEqual(value, t._attrs[key])
        self.assertTrue(key in t._dirty)

    def test__setitem__existing_item_unchanged(self):

        class Test(resource.Resource):
            pass

        key = "attr"
        value = 1
        t = Test(attrs={key: value})
        t._reset_dirty()  # Clear dirty list so this checks as unchanged.
        t[key] = value

        self.assertEqual(value, t._attrs[key])
        self.assertTrue(key not in t._dirty)

    def test__setitem__new_item(self):

        class Test(resource.Resource):
            pass

        t = Test()
        key = "attr"
        value = 1
        t[key] = value

        self.assertEqual(value, t._attrs[key])
        self.assertTrue(key in t._dirty)

    def test__delitem__(self):

        class Test(resource.Resource):
            pass

        key = "attr"
        value = 1
        t = Test(attrs={key: value})

        del t[key]

        self.assertTrue(key not in t._attrs)
        self.assertTrue(key in t._dirty)

    def test__len__(self):

        class Test(resource.Resource):
            pass

        attrs = {"a": 1, "b": 2, "c": 3}
        t = Test(attrs=attrs)

        self.assertEqual(len(attrs.keys()), len(t))

    def test__iter__(self):

        class Test(resource.Resource):
            pass

        attrs = {"a": 1, "b": 2, "c": 3}
        t = Test(attrs=attrs)

        for attr in t:
            self.assertEqual(attrs[attr], t[attr])

    def _test_resource_serialization(self, session_method, resource_method):
        attr_type = resource.Resource

        class Test(resource.Resource):
            allow_create = True
            attr = resource.prop("attr", type=attr_type)

        the_id = 123
        sot = Test()
        sot.attr = resource.Resource({"id": the_id})
        self.assertEqual(attr_type, type(sot.attr))

        def fake_call(*args, **kwargs):
            attrs = kwargs["json"]
            try:
                json.dumps(attrs)
            except TypeError as e:
                self.fail("Unable to serialize _attrs: %s" % e)
            resp = mock.Mock()
            resp.json = mock.Mock(return_value=attrs)
            return resp

        session = mock.Mock()
        setattr(session, session_method, mock.Mock(side_effect=fake_call))

        if resource_method == "create_by_id":
            session.create_by_id(session, sot._attrs)
        elif resource_method == "update_by_id":
            session.update_by_id(session, None, sot._attrs)

    def test_create_serializes_resource_types(self):
        self._test_resource_serialization("post", "create_by_id")

    def test_update_serializes_resource_types(self):
        self._test_resource_serialization("patch", "update_by_id")


class FakeResponse(object):
    def __init__(self, response):
        self.body = response

    def json(self):
        return self.body


class TestFind(base.TestCase):
    NAME = 'matrix'
    ID = 'Fishburne'
    PROP = 'attribute2'

    def setUp(self):
        super(TestFind, self).setUp()
        self.mock_session = mock.Mock()
        self.mock_get = mock.Mock()
        self.mock_session.get = self.mock_get
        self.matrix = {'id': self.ID, 'name': self.NAME, 'prop': self.PROP}

    def test_name(self):
        self.mock_get.side_effect = [
            exceptions.NotFoundException(),
            FakeResponse({FakeResource.resources_key: [self.matrix]})
        ]

        result = FakeResource.find(self.mock_session, self.NAME,
                                   path_args=fake_arguments)

        self.assertEqual(self.NAME, result.name)
        self.assertEqual(self.PROP, result.prop)

    def test_id(self):
        self.mock_get.side_effect = [
            FakeResponse({FakeResource.resource_key: self.matrix})
        ]

        result = FakeResource.find(self.mock_session, self.ID,
                                   path_args=fake_arguments)

        self.assertEqual(self.ID, result.id)
        self.assertEqual(self.PROP, result.prop)

        path = "fakes/" + fake_parent + "/data/" + self.ID
        self.mock_get.assert_any_call(path, endpoint_filter=None)

    def test_id_no_retrieve(self):
        self.mock_get.side_effect = [
            FakeResponse({FakeResource.resources_key: [self.matrix]})
        ]

        class NoRetrieveResource(FakeResource):
            allow_retrieve = False

        result = NoRetrieveResource.find(self.mock_session, self.ID,
                                         path_args=fake_arguments)

        self.assertEqual(self.ID, result.id)
        self.assertEqual(self.PROP, result.prop)

    def test_dups(self):
        dupe = self.matrix.copy()
        dupe['id'] = 'different'
        self.mock_get.side_effect = [
            # Raise a 404 first so we get out of the ID search and into name.
            exceptions.NotFoundException(),
            FakeResponse({FakeResource.resources_key: [self.matrix, dupe]})
        ]

        self.assertRaises(exceptions.DuplicateResource, FakeResource.find,
                          self.mock_session, self.NAME)

    def test_id_attribute_find(self):
        floater = {'ip_address': "127.0.0.1", 'prop': self.PROP}
        self.mock_get.side_effect = [
            FakeResponse({FakeResource.resource_key: floater})
        ]

        FakeResource.id_attribute = 'ip_address'
        FakeResource.id_attribute = 'ip_address'
        result = FakeResource.find(self.mock_session, "127.0.0.1",
                                   path_args=fake_arguments)
        self.assertEqual("127.0.0.1", result.id)
        self.assertEqual(self.PROP, result.prop)

        FakeResource.id_attribute = 'id'

        p = {'ip_address': "127.0.0.1"}
        path = fake_path + "?limit=2"
        self.mock_get.called_once_with(path, params=p, endpoint_filter=None)

    def test_nada(self):
        self.mock_get.side_effect = [
            exceptions.NotFoundException(),
            FakeResponse({FakeResource.resources_key: []})
        ]

        self.assertIsNone(FakeResource.find(self.mock_session, self.NAME))

    def test_no_name(self):
        self.mock_get.side_effect = [
            exceptions.NotFoundException(),
            FakeResponse({FakeResource.resources_key: [self.matrix]})
        ]
        FakeResource.name_attribute = None

        self.assertIsNone(FakeResource.find(self.mock_session, self.NAME))

    def test_nada_not_ignored(self):
        self.mock_get.side_effect = [
            exceptions.NotFoundException(),
            FakeResponse({FakeResource.resources_key: []})
        ]

        self.assertRaises(exceptions.ResourceNotFound, FakeResource.find,
                          self.mock_session, self.NAME, ignore_missing=False)


class TestWaitForStatus(base.TestCase):

    def __init__(self, *args, **kwargs):
        super(TestWaitForStatus, self).__init__(*args, **kwargs)
        self.build = FakeResponse(self.body_with_status(fake_body, 'BUILD'))
        self.active = FakeResponse(self.body_with_status(fake_body, 'ACTIVE'))
        self.error = FakeResponse(self.body_with_status(fake_body, 'ERROR'))

    def setUp(self):
        super(TestWaitForStatus, self).setUp()
        self.sess = mock.Mock()

    def body_with_status(self, body, status):
        body_copy = copy.deepcopy(body)
        body_copy[fake_resource]['status'] = status
        return body_copy

    def test_wait_for_status_nothing(self):
        self.sess.get = mock.Mock()
        sot = FakeResource.new(**fake_data)
        sot.status = 'ACTIVE'

        self.assertEqual(sot, resource.wait_for_status(
            self.sess, sot, 'ACTIVE', [], 1, 2))
        self.assertEqual([], self.sess.get.call_args_list)

    def test_wait_for_status(self):
        self.sess.get = mock.Mock()
        self.sess.get.side_effect = [self.build, self.active]
        sot = FakeResource.new(**fake_data)

        self.assertEqual(sot, resource.wait_for_status(
            self.sess, sot, 'ACTIVE', [], 1, 2))

    def test_wait_for_status_timeout(self):
        self.sess.get = mock.Mock()
        self.sess.get.side_effect = [self.build, self.build]
        sot = FakeResource.new(**fake_data)

        self.assertRaises(exceptions.ResourceTimeout, resource.wait_for_status,
                          self.sess, sot, 'ACTIVE', ['ERROR'], 1, 2)

    def test_wait_for_status_failures(self):
        self.sess.get = mock.Mock()
        self.sess.get.side_effect = [self.build, self.error]
        sot = FakeResource.new(**fake_data)

        self.assertRaises(exceptions.ResourceFailure, resource.wait_for_status,
                          self.sess, sot, 'ACTIVE', ['ERROR'], 1, 2)

    def test_wait_for_status_no_status(self):
        class FakeResourceNoStatus(resource.Resource):
            allow_retrieve = True

        sot = FakeResourceNoStatus.new(id=123)

        self.assertRaises(AttributeError, resource.wait_for_status,
                          self.sess, sot, 'ACTIVE', ['ERROR'], 1, 2)


class TestWaitForDelete(base.TestCase):

    def test_wait_for_delete(self):
        sess = mock.Mock()
        sot = FakeResource.new(**fake_data)
        sot.get = mock.Mock()
        sot.get.side_effect = [
            sot,
            exceptions.NotFoundException()]

        self.assertEqual(sot, resource.wait_for_delete(sess, sot, 1, 2))

    def test_wait_for_delete_fail(self):
        sess = mock.Mock()
        sot = FakeResource.new(**fake_data)
        sot.get = mock.Mock(return_value=sot)

        self.assertRaises(exceptions.ResourceTimeout, resource.wait_for_delete,
                          sess, sot, 1, 2)



# Licensed under the Apache License, Version 2.0 (the "License"); you may

# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy

from ecl.object_store import object_store_service
from ecl.object_store.v1 import _base
from ecl import resource


class Object(_base.BaseResource):
    _custom_metadata_prefix = "X-Object-Meta-"
    _system_metadata = {
        "content_disposition": "content-disposition",
        "content_encoding": "content-encoding",
        "content_type": "content-type",
        "delete_after": "x-delete-after",
        "delete_at": "x-delete-at",
        "is_content_type_detected": "x-detect-content-type",
    }

    base_path = "/%(container)s"
    service = object_store_service.ObjectStoreService()
    id_attribute = "name"

    allow_create = True
    allow_retrieve = True
    allow_update = True
    allow_delete = True
    allow_list = True
    allow_head = True

    # Data to be passed during a POST call to create an object on the server.
    data = None

    # URL parameters
    #: The unique name for the container.
    container = resource.prop("container")
    #: The unique name for the object.
    name = resource.prop("name")

    # Object details
    hash = resource.prop("hash")
    bytes = resource.prop("bytes")

    # Headers for HEAD and GET requests
    #: If set to True, Object Storage queries all replicas to return
    #: the most recent one. If you omit this header, Object Storage
    #: responds faster after it finds one valid replica. Because
    #: setting this header to True is more expensive for the back end,
    #: use it only when it is absolutely needed. *Type: bool*
    is_newest = resource.header("x-newest", type=bool)
    #: TODO(briancurtin) there's a lot of content here...
    range = resource.header("range", type=dict)
    #: See http://www.ietf.org/rfc/rfc2616.txt.
    if_match = resource.header("if-match", type=dict)
    #: In combination with Expect: 100-Continue, specify an
    #: "If-None-Match: \*" header to query whether the server already
    #: has a copy of the object before any data is sent.
    if_none_match = resource.header("if-none-match", type=dict)
    #: See http://www.ietf.org/rfc/rfc2616.txt.
    if_modified_since = resource.header("if-modified-since", type=dict)
    #: See http://www.ietf.org/rfc/rfc2616.txt.
    if_unmodified_since = resource.header("if-unmodified-since", type=dict)

    # Query parameters
    #: Used with temporary URLs to sign the request. For more
    #: information about temporary URLs, see OpenStack Object Storage
    #: API v1 Reference.
    signature = resource.header("signature")
    #: Used with temporary URLs to specify the expiry time of the
    #: signature. For more information about temporary URLs, see
    #: OpenStack Object Storage API v1 Reference.
    expires_at = resource.header("expires")
    #: If you include the multipart-manifest=get query parameter and
    #: the object is a large object, the object contents are not
    #: returned. Instead, the manifest is returned in the
    #: X-Object-Manifest response header for dynamic large objects
    #: or in the response body for static large objects.
    multipart_manifest = resource.header("multipart-manifest")

    # Response headers from HEAD and GET
    #: HEAD operations do not return content. However, in this
    #: operation the value in the Content-Length header is not the
    #: size of the response body. Instead it contains the size of
    #: the object, in bytes.
    content_length = resource.header("content-length")
    #: The MIME type of the object.
    content_type = resource.header("content-type")
    #: The type of ranges that the object accepts.
    accept_ranges = resource.header("accept-ranges")
    #: For objects smaller than 5 GB, this value is the MD5 checksum
    #: of the object content. The value is not quoted.
    #: For manifest objects, this value is the MD5 checksum of the
    #: concatenated string of MD5 checksums and ETags for each of
    #: the segments in the manifest, and not the MD5 checksum of
    #: the content that was downloaded. Also the value is enclosed
    #: in double-quote characters.
    #: You are strongly recommended to compute the MD5 checksum of
    #: the response body as it is received and compare this value
    #: with the one in the ETag header. If they differ, the content
    #: was corrupted, so retry the operation.
    etag = resource.header("etag")
    #: Set to True if this object is a static large object manifest object.
    #: *Type: bool*
    is_static_large_object = resource.header("x-static-large-object",
                                             type=bool)
    #: If set, the value of the Content-Encoding metadata.
    #: If not set, this header is not returned by this operation.
    content_encoding = resource.header("content-encoding")
    #: If set, specifies the override behavior for the browser.
    #: For example, this header might specify that the browser use
    #: a download program to save this file rather than show the file,
    #: which is the default.
    #: If not set, this header is not returned by this operation.
    content_disposition = resource.header("content-disposition")
    #: Specifies the number of seconds after which the object is
    #: removed. Internally, the Object Storage system stores this
    #: value in the X-Delete-At metadata item.
    delete_after = resource.header("x-delete-after")
    #: If set, the time when the object will be deleted by the system
    #: in the format of a UNIX Epoch timestamp.
    #: If not set, this header is not returned by this operation.
    delete_at = resource.header("x-delete-at")
    #: If set, to this is a dynamic large object manifest object.
    #: The value is the container and object name prefix of the
    #: segment objects in the form container/prefix.
    object_manifest = resource.header("x-object-manifest")
    #: The timestamp of the transaction.
    timestamp = resource.header("x-timestamp")
    #: The date and time that the object was created or the last
    #: time that the metadata was changed.
    last_modified_at = resource.header("last_modified", alias="last-modified")

    # Headers for PUT and POST requests
    #: Set to chunked to enable chunked transfer encoding. If used,
    #: do not set the Content-Length header to a non-zero value.
    transfer_encoding = resource.header("transfer-encoding")
    #: If set to true, Object Storage guesses the content type based
    #: on the file extension and ignores the value sent in the
    #: Content-Type header, if present. *Type: bool*
    is_content_type_detected = resource.header("x-detect-content-type",
                                               type=bool)
    #: If set, this is the name of an object used to create the new
    #: object by copying the X-Copy-From object. The value is in form
    #: {container}/{object}. You must UTF-8-encode and then URL-encode
    #: the names of the container and object before you include them
    #: in the header.
    #: Using PUT with X-Copy-From has the same effect as using the
    #: COPY operation to copy an object.
    copy_from = resource.header("x-copy-from")

    # The Object Store treats the metadata for its resources inconsistently so
    # Object.set_metadata must override the BaseResource.set_metadata to
    # account for it.
    def set_metadata(self, session, metadata):
        # Filter out items with empty values so the create metadata behaviour
        # is the same as account and container
        filtered_metadata = \
            {key: value for key, value in metadata.iteritems() if value}

        # Get a copy of the original metadata so it doesn't get erased on POST
        # and update it with the new metadata values.
        obj = self.head(session)
        metadata2 = copy.deepcopy(obj.metadata)
        metadata2.update(filtered_metadata)

        # Include any original system metadata so it doesn't get erased on POST
        for key in self._system_metadata:
            value = getattr(obj, key)
            if value and key not in metadata2:
                metadata2[key] = value

        super(Object, self).set_metadata(session, metadata2)

    # The Object Store treats the metadata for its resources inconsistently so
    # Object.delete_metadata must override the BaseResource.delete_metadata to
    # account for it.
    def delete_metadata(self, session, keys):
        # Get a copy of the original metadata so it doesn't get erased on POST
        # and update it with the new metadata values.
        obj = self.head(session)
        metadata = copy.deepcopy(obj.metadata)

        # Include any original system metadata so it doesn't get erased on POST
        for key in self._system_metadata:
            value = getattr(obj, key)
            if value:
                metadata[key] = value

        # Remove the metadata
        for key in keys:
            if key == 'delete_after':
                del(metadata['delete_at'])
            else:
                del(metadata[key])

        url = self._get_url(self, self.id)
        session.post(url, endpoint_filter=self.service,
                     headers=self._calculate_headers(metadata))

    def get(self, session, include_headers=False, args=None):
        url = self._get_url(self, self.id)
        headers = {'Accept': 'bytes'}
        resp = session.get(url, endpoint_filter=self.service, headers=headers)
        resp = resp.content
        self._set_metadata()
        return resp

    def create(self, session):
        url = self._get_url(self, self.id)

        headers = self.get_headers()
        headers['Accept'] = ''
        if self.data is not None:
            resp = session.put(url, endpoint_filter=self.service,
                               data=self.data,
                               headers=headers).headers
        else:
            resp = session.post(url, endpoint_filter=self.service, data=None,
                                headers=headers).headers
        self.set_headers(resp)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.telemetry.v2 import _proxy
from ecl.telemetry.v2 import alarm
from ecl.telemetry.v2 import alarm_change
from ecl.telemetry.v2 import capability
from ecl.telemetry.v2 import meter
from ecl.telemetry.v2 import resource
from ecl.telemetry.v2 import sample
from ecl.telemetry.v2 import statistics
from ecl.tests.unit import test_proxy_base


class TestTelemetryProxy(test_proxy_base.TestProxyBase):
    def setUp(self):
        super(TestTelemetryProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)

    def test_alarm_change_find(self):
        self.verify_find(self.proxy.find_alarm_change,
                         alarm_change.AlarmChange)

    def test_alarm_changes(self):
        larm = alarm.Alarm.existing(alarm_id='larm')
        expected_kwargs = {'path_args': {'alarm_id': 'larm'}}
        self.verify_list(self.proxy.alarm_changes, alarm_change.AlarmChange,
                         method_args=[larm], paginated=False,
                         expected_kwargs=expected_kwargs)

    def test_alarm_create_attrs(self):
        self.verify_create(self.proxy.create_alarm, alarm.Alarm)

    def test_alarm_delete(self):
        self.verify_delete(self.proxy.delete_alarm, alarm.Alarm, False)

    def test_alarm_delete_ignore(self):
        self.verify_delete(self.proxy.delete_alarm, alarm.Alarm, True)

    def test_alarm_find(self):
        self.verify_find(self.proxy.find_alarm, alarm.Alarm)

    def test_alarm_get(self):
        self.verify_get(self.proxy.get_alarm, alarm.Alarm)

    def test_alarms(self):
        self.verify_list(self.proxy.alarms, alarm.Alarm, paginated=False)

    def test_alarm_update(self):
        self.verify_update(self.proxy.update_alarm, alarm.Alarm)

    def test_capability_find(self):
        self.verify_find(self.proxy.find_capability, capability.Capability)

    def test_capabilities(self):
        self.verify_list(self.proxy.capabilities, capability.Capability,
                         paginated=False)

    def test_meter_find(self):
        self.verify_find(self.proxy.find_meter, meter.Meter)

    def test_meters(self):
        self.verify_list(self.proxy.meters, meter.Meter, paginated=False)

    def test_resource_find(self):
        self.verify_find(self.proxy.find_resource, resource.Resource)

    def test_resource_get(self):
        self.verify_get(self.proxy.get_resource, resource.Resource)

    def test_resources(self):
        self.verify_list(self.proxy.resources, resource.Resource,
                         paginated=False)

    def test_sample_create_attrs(self):
        self.verify_create(self.proxy.create_sample, sample.Sample)

    def test_sample_find(self):
        self.verify_find(self.proxy.find_sample, sample.Sample)

    def test_samples(self):
        met = meter.Meter.existing(name='meterone')
        expected_kwargs = {'path_args': {'counter_name': 'meterone'}}
        self.verify_list(self.proxy.samples, sample.Sample,
                         method_args=[met],
                         paginated=False, expected_kwargs=expected_kwargs)

    def test_statistics_find(self):
        self.verify_find(self.proxy.find_statistics, statistics.Statistics)

    def test_statistics(self):
        met = meter.Meter.existing(name='meterone')
        expected_kwargs = {'path_args': {'meter_name': 'meterone'}}
        self.verify_list(self.proxy.statistics, statistics.Statistics,
                         method_args=[met],
                         paginated=False, expected_kwargs=expected_kwargs)



"""
Exception definitions for Interconnectivity.
"""
from ecl import exceptions


class HttpException(exceptions.HttpException):
    api_error_key = 'reason'


class NotFoundException(HttpException):
    pass



# -*- coding: utf-8 -*-

import six

from ecl.tests.functional import base


class TestAvailabilityZone(base.BaseFunctionalTest):

    @classmethod
    def setUpClass(cls):
        super(TestAvailabilityZone, cls).setUpClass()

    def test_azs(self):
        azs = self.conn.compute.availability_zones()
        self.assertGreater(len(azs), 0)

        for az in azs:
            self.assertIsInstance(az.name, six.string_types)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import six
import time
from ecl.tests.functional import base

class TestVolume(base.BaseFunctionalTest):
    def test_01_volumes(self):
        volumes = list(self.conn.storage.volumes(details=True))
        volume = volumes[1]
        self.assertIsInstance(volume.id, six.string_types)
        self.assertIsInstance(volume.status, six.string_types)
        self.assertIsInstance(volume.name, six.string_types)
        self.assertIsInstance(volume.size, int)
        self.assertIsInstance(volume.description, six.string_types)
        self.assertIsInstance(volume.iops_per_gb, six.string_types)
        self.assertIsInstance(volume.initiator_iqns, list)
        self.assertIsInstance(volume.target_ips, list)
        self.assertIsInstance(volume.metadata, dict)
        self.assertIsInstance(volume.virtual_storage_id, six.string_types)
        self.assertIsInstance(volume.availability_zone, six.string_types)
        self.assertIsInstance(volume.created_at, six.string_types)
        self.assertIsInstance(volume.updated_at, six.string_types)
        self.assertIsInstance(volume.id, six.string_types)

    def test_02_show_volume(self):
        volume = self.conn.storage.get_volume("4096336b-7035-412a-8148-ad999f0e0bc8")
        self.assertIsInstance(volume.id, six.string_types)
        self.assertIsInstance(volume.status, six.string_types)
        self.assertIsInstance(volume.name, six.string_types)
        self.assertIsInstance(volume.size, int)
        self.assertIsInstance(volume.description, six.string_types)
        self.assertIsInstance(volume.iops_per_gb, six.string_types)
        self.assertIsInstance(volume.initiator_iqns, list)
        self.assertIsInstance(volume.target_ips, list)
        self.assertIsInstance(volume.metadata, dict)
        self.assertIsInstance(volume.virtual_storage_id, six.string_types)
        self.assertIsInstance(volume.availability_zone, six.string_types)
        self.assertIsInstance(volume.created_at, six.string_types)
        self.assertIsInstance(volume.updated_at, six.string_types)
        self.assertIsInstance(volume.id, six.string_types)

    def test_03_update_volume(self):
        volume = self.conn.storage.update_volume(
            "4096336b-7035-412a-8148-ad999f0e0bc8",
            description="updated_test"
        )
        print volume.description
        self.assertIsInstance(volume.id, six.string_types)
        self.assertIsInstance(volume.status, six.string_types)
        self.assertIsInstance(volume.name, six.string_types)
        self.assertIsInstance(volume.size, int)
        self.assertIsInstance(volume.description, six.string_types)
        self.assertIsInstance(volume.iops_per_gb, six.string_types)
        self.assertIsInstance(volume.initiator_iqns, list)
        self.assertIsInstance(volume.target_ips, list)
        self.assertIsInstance(volume.metadata, dict)
        self.assertIsInstance(volume.virtual_storage_id, six.string_types)
        self.assertIsInstance(volume.availability_zone, six.string_types)
        self.assertIsInstance(volume.created_at, six.string_types)
        self.assertIsInstance(volume.updated_at, six.string_types)
        self.assertIsInstance(volume.id, six.string_types)

    @classmethod
    def test_04_create_volume(cls):
        volume = cls.conn.storage.create_volume(
            name="sdk_test",
            size=100,
            virtual_storage_id="19ff4cba-3b86-4da1-9663-25ea74f9b0a9",
        )
        assert isinstance(volume.id, six.string_types)
        assert isinstance(volume.status, six.string_types)
        assert isinstance(volume.name, six.string_types)
        assert isinstance(volume.size, int)
        assert isinstance(volume.description, six.string_types)
        assert isinstance(volume.iops_per_gb, six.string_types)
        assert isinstance(volume.initiator_iqns, list)
        assert isinstance(volume.target_ips, list)
        assert isinstance(volume.metadata, dict)
        assert isinstance(volume.virtual_storage_id, six.string_types)
        assert isinstance(volume.availability_zone, six.string_types)
        assert isinstance(volume.created_at, six.string_types)
        cls.vol_id = volume.id

    def test_05_delete_volume(self):
        time.sleep(20)
        volume = self.conn.storage.delete_volume(self.vol_id)



"""
Exception definitions for Provider Connectivity
"""
import json
from ecl import exceptions


class HttpException(exceptions.HttpException):
    api_error_key = 'cause'


class NotFoundException(HttpException):
    pass



# -*- coding: utf-8 -*-


from ecl.block_store import block_store_service
from ecl import resource


class AvailabilityZone(resource.Resource):
    resources_key = 'availabilityZoneInfo'
    base_path = '/os-availability-zone'

    service = block_store_service.BlockStoreService()

    # capabilities
    allow_list = True

    id_attribute = "name"
    # Properties
    #: name of availability zone
    name = resource.prop('zoneName')
    #: state of availability zone
    state = resource.prop('zoneState')
    #: hosts of availability zone
    hosts = resource.prop('hosts')



# -*- coding: utf-8 -*-

from ecl.security_order.v1 import device as _fgs
from ecl.security_order.v1 import ha_device as _fgha
from ecl.security_order.v1 import waf as _fgwaf
from ecl.security_order.v1 import host_based_security as _hbs
from ecl import proxy2


class Proxy(proxy2.BaseProxy):

    def devices(self, locale=None):
        """List Managed Firwall/UTM devices of single constitution.

        :param string locale: Messages are displayed in Japanese or English
                              depending on this value.
                              ja: Japanese, en: English. Default value is "en".
        :return: Single Firwall/UTM.
        :rtype: :class:`~ecl.security.v1.device.Device`
        """
        fgs = _fgs.Device()
        return fgs.list(self.session, locale=locale)

    def create_device(self, operatingmode, licensekind,
                               azgroup, locale=None):
        """Create a new Managed Firewall/UTM device of single constitution.

        :param string operatingmode: Set "FW" or "UTM" to this value.
        :param string licensekind: Set "02" or "08" as FW/UTM plan.
        :param string azgroup: Availability Zone
        :param string locale: Messages are displayed in Japanese or English
                              depending on this value.
                              ja: Japanese, en: English. Default value is "en".
        :return: Single Firwall/UTM.
        :rtype: :class:`~ecl.security.v1.device.Device`
        """
        body = {}
        body["tenant_id"] = self.session.get_project_id()
        body["gt_host"] = [{
            "operatingmode": operatingmode,
            "licensekind": licensekind,
            "azgroup": azgroup
        }]
        body["sokind"] = "A"
        if locale:
            body["locale"] = locale
        return self._create(_fgs.Device, **body)

    def update_device(self, hostname, operatingmode,
                               licensekind, locale=None):
        """Change menu (Firewall/Managed UTM) and/or plan of single device.

        :param string operatingmode: Set "FW" or "UTM" to this value.
        :param string licensekind: Set "02" or "08" as FW/UTM plan.
        :param string hostname: Set the hostname.
        :param string locale: Messages are displayed in Japanese or English
                              depending on this value.
                              ja: Japanese, en: English. Default value is "en".
        :return: Single Firwall/UTM.
        :rtype: :class:`~ecl.security.v1.device.Device`
        """
        body = {}
        body["tenant_id"] = self.session.get_project_id()
        body["gt_host"] = [{
            "hostname": hostname,
            "operatingmode": operatingmode,
            "licensekind": licensekind
        }]
        body["sokind"] = "M"
        if locale:
            body.update({"locale": locale})
        fgs = _fgs.Device()
        return fgs.update(self.session, **body)

    def delete_device(self, hostname, locale=None):
        """Delete a Managed Firewall/UTM device of single constitution.

        :param string hostname: Set the hostname.
        :param string locale: Messages are displayed in Japanese or English
                              depending on this value.
                              ja: Japanese, en: English. Default value is "en".
        :return: Single Firwall/UTM.
        :rtype: :class:`~ecl.security.v1.device.Device`
        """
        body = {}
        body["tenant_id"] = self.session.get_project_id()
        body["gt_host"] = [{
            "hostname": hostname
        }]
        body["sokind"] = "D"
        if locale:
            body["locale"] = locale
        fgs = _fgs.Device()
        return fgs.delete(self.session, body, locale=locale)

    def ha_devices(self, locale=None):
        """List Managed Firwall/UTM devices of single constitution.

        :param string locale: Messages are displayed in Japanese or English
                              depending on this value.
                              ja: Japanese, en: English. Default value is "en".
        :return: HA Firwall/UTM.
        :rtype: :class:`~ecl.security.v1.ha_device.HADevice`
        """
        fgha = _fgha.HADevice()
        return fgha.list(self.session, locale=locale)

    def create_ha_device(self, operatingmode, licensekind,
                           azgroup1, azgroup2,
                           halink1networkid, halink1subnetid,
                           halink1ipaddress1, halink1ipaddress2,
                           halink2networkid, halink2subnetid,
                           halink2ipaddress1, halink2ipaddress2,
                           locale=None):
        """Create a new Managed Firewall/UTM device of single constitution.

        :param string operatingmode: Set "UTM_HA" or "FW_HA" to this value.
        :param string licensekind: Set "02" or "08" as FW/UTM plan.
        :param string azgroup1: Availability Zone
        :param string azgroup2: Availability Zone
        :param string halink1networkid: Set Network ID to be used for HA line.
        :param string halink1subnetid: Set Subnet ID to be used for HA line.
        :param string halink1ipaddress1: Set value of IPv4.
        :param string halink1ipaddress2: Set value of IPv4.
        :param string halink2networkid: Set Network ID to be used for HA line.
        :param string halink2subnetid: Set Subnet ID to be used for HA line.
        :param string halink2ipaddress1: Set value of IPv4.
        :param string halink2ipaddress2: Set value of IPv4.
        :param string locale: Messages are displayed in Japanese or English
                              depending on this value.
                              ja: Japanese, en: English. Default value is "en".
        :return: HA Firwall/UTM.
        :rtype: :class:`~ecl.security.v1.ha_device.HADevice`
        """
        body = {}
        body["tenant_id"] = self.session.get_project_id()
        body["gt_host"] = [{
            "operatingmode": operatingmode,
            "licensekind": licensekind,
            "azgroup": azgroup1,
            "halink1networkid": halink1networkid,
            "halink1subnetid": halink1subnetid,
            "halink1ipaddress": halink1ipaddress1,
            "halink2networkid": halink2networkid,
            "halink2subnetid": halink2subnetid,
            "halink2ipaddress": halink2ipaddress1
        },{
            "operatingmode": operatingmode,
            "licensekind": licensekind,
            "azgroup": azgroup2,
            "halink1networkid": halink1networkid,
            "halink1subnetid": halink1subnetid,
            "halink1ipaddress": halink1ipaddress2,
            "halink2networkid": halink2networkid,
            "halink2subnetid": halink2subnetid,
            "halink2ipaddress": halink2ipaddress2
        }]
        body["sokind"] = "AH"
        if locale:
            body["locale"] = locale
        return self._create(_fgha.HADevice, **body)

    def update_ha_device(self, hostname1, hostname2, operatingmode,
                           licensekind, locale=None):
        """Change menu (Firewall/Managed UTM) and/or plan of single device.

        :param string hostname1: Set the hostname.
        :param string hostname2: Set the hostname.
        :param string operatingmode: Set "UTM_HA" or "FW_HA" to this value.
        :param string licensekind: Set "02" or "08" as FW/UTM plan.
        :param string locale: Messages are displayed in Japanese or English
                              depending on this value.
                              ja: Japanese, en: English. Default value is "en".
        :return: HA Firwall/UTM.
        :rtype: :class:`~ecl.security.v1.ha_device.HADevice`
        """
        body = {}
        body["tenant_id"] = self.session.get_project_id()
        body["gt_host"] = [{
            "hostname": hostname1,
            "operatingmode": operatingmode,
            "licensekind": licensekind
        },{
            "hostname": hostname2,
            "operatingmode": operatingmode,
            "licensekind": licensekind
        }]
        body["sokind"] = "MH"
        if locale:
            body.update({"locale": locale})
        fgha = _fgha.HADevice()
        return fgha.update(self.session, **body)

    def delete_ha_device(self, hostname1, hostname2, locale=None):
        """Delete a Managed Firewall/UTM device of single constitution.

        :param string hostname1: Set the hostname.
        :param string hostname2: Set the hostname.
        :param string locale: Messages are displayed in Japanese or English
                              depending on this value.
                              ja: Japanese, en: English. Default value is "en".
        :return: HA Firwall/UTM.
        :rtype: :class:`~ecl.security.v1.ha_device.HADevice`
        """
        body = {}
        body["tenant_id"] = self.session.get_project_id()
        body["gt_host"] = [{
            "hostname": hostname1
        }, {
            "hostname": hostname2
        }]
        body["sokind"] = "DH"
        if locale:
            body["locale"] = locale
        fgha = _fgha.HADevice()
        return fgha.delete(self.session, body, locale=locale)

    def get_device_order_status(self, soid, locale=None):
        """Check progress status of Managed Firewall/UTM device Service Order.

        :param string soid: This value is returned value of when you execute
                            Create Server, Update Server or Delete Server API.
        :param string locale: Messages are displayed in Japanese or English
                              depending on this value.
                              ja: Japanese, en: English. Default value is "en".
        :return: Single Firwall/UTM.
        :rtype: :class:`~ecl.security.v1.device.Device`
        """
        fgs = _fgs.Device()
        return fgs.get_order_status(self.session, soid, locale=locale)

    def wafs(self, locale=None):
        """List active waf devices you ordered.

        :param string locale: Messages are displayed in Japanese or English
                              depending on this value.
                              ja: Japanese, en: English. Default value is "en".
        :return: WAF.
        :rtype: :class:`~ecl.security.v1.waf.WAF`
        """
        fgwaf = _fgwaf.WAF()
        return fgwaf.list(self.session, locale=locale)

    def create_waf(self, licensekind, azgroup, locale=None):
        """Create a new WAF device.

        :param string licensekind: Set "02", "04" or "08" as WAF plan.
        :param string azgroup: Availability Zone
        :param string locale: Messages are displayed in Japanese or English
                              depending on this value.
                              ja: Japanese, en: English. Default value is "en".
        :return: WAF.
        :rtype: :class:`~ecl.security.v1.waf.WAF`
        """
        body = {}
        body["tenant_id"] = self.session.get_project_id()
        body["gt_host"] = [{
            "operatingmode": "WAF",
            "licensekind": licensekind,
            "azgroup": azgroup
        }]
        body["sokind"] = "A"
        if locale:
            body["locale"] = locale
        return self._create(_fgwaf.WAF, **body)

    def get_waf_order_status(self, soid, locale=None):
        """Check progress status of Managed WAF device Service Order.

        :param string soid: This value is returned value of when you execute
                            Create Server, Update Server or Delete Server API.
        :param string locale: Messages are displayed in Japanese or English
                              depending on this value.
                              ja: Japanese, en: English. Default value is "en".
        :return: WAF.
        :rtype: :class:`~ecl.security.v1.waf.WAF`
        """
        fgwaf = _fgwaf.WAF()
        return fgwaf.get_order_status(self.session, soid, locale=locale)

    def update_waf(self, hostname, licensekind, locale=None):
        """Change plan of device.

        :param string licensekind: Set "02", "04" or "08" as WAF plan.
        :param string hostname: Set the hostname.
        :param string locale: Messages are displayed in Japanese or English
                              depending on this value.
                              ja: Japanese, en: English. Default value is "en".
        :return: WAF.
        :rtype: :class:`~ecl.security.v1.waf.WAF`
        """
        body = {}
        body["tenant_id"] = self.session.get_project_id()
        body["gt_host"] = [{
            "hostname": hostname,
            "licensekind": licensekind
        }]
        body["sokind"] = "M"
        if locale:
            body.update({"locale": locale})
        fgwaf = _fgwaf.WAF()
        return fgwaf.update(self.session, **body)

    def delete_waf(self, hostname, locale=None):
        """Delete a WAF device.

        :param string hostname: Set the hostname.
        :param string locale: Messages are displayed in Japanese or English
                              depending on this value.
                              ja: Japanese, en: English. Default value is "en".
        :return: WAF.
        :rtype: :class:`~ecl.security.v1.waf.WAF`
        """
        body = {}
        body["tenant_id"] = self.session.get_project_id()
        body["gt_host"] = [{
            "hostname": hostname
        }]
        body["sokind"] = "D"
        if locale:
            body["locale"] = locale
        fgwaf = _fgwaf.WAF()
        return fgwaf.delete(self.session, body, locale=locale)

    def get_hbs_order_status(self, soid, locale=None):
        """Check progress status of Host-based Security Service Order.

        :param string soid: This value is returned value of when you execute API
                            of Order Host-based Security, Change menu or
                            quantity, or Cancel the order.
        :param string locale: Messages are displayed in Japanese or English
                              depending on this value.
                              ja: Japanese, en: English. Default value is "en".
        :return: Host Based Security.
        :rtype: :class:`~ecl.security.v1.host_based_security.HostBasedSecurity`
        """
        hbs = _hbs.HostBasedSecurity()
        return hbs.get_order_status(self.session, soid, locale=locale)

    def get_hbs_order_info(self, locale=None):
        """Get Order Information that tied to tenant id.

        :param string locale: Messages are displayed in Japanese or English
                              depending on this value.
                              ja: Japanese, en: English. Default value is "en".
        :return: Host Based Security.
        :rtype: :class:`~ecl.security.v1.host_based_security.HostBasedSecurity`
        """
        hbs = _hbs.HostBasedSecurity()
        return hbs.get_order_info(self.session, locale=locale)

    def order_hbs(self, service_order_service, max_agent_value,
                                  mailaddress, dsm_lang, time_zone,
                                  locale=None):
        """Make a new application for Host-based Security.

        :param string service_order_service: Requested menu.
            Set "Managed Anti-Virus", "Managed Virtual Patch"
            or "Managed Host-based Security Package" to this field.
        :param string max_agent_value: Set maximum quantity of Agenet usage.
        :param string mailaddress: Contactable mail address.
        :param string dsm_lang: This value is used for language of Deep
                                Security Manager. ja: Japanese, en: English.
        :param string time_zone: Set "Asia/Tokyo" for JST or "Etc/GMT" for UTC.
        :param string locale: Messages are displayed in Japanese or English
                              depending on this value.
                              ja: Japanese, en: English. Default value is "en".
        :return: Host Based Security
        :rtype: :class:`~ecl.security.v1.host_based_security.HostBasedSecurity`
        """
        body = {}
        body["tenant_id"] = self.session.get_project_id()
        body["service_order_service"] = service_order_service
        body["max_agent_value"] = max_agent_value
        body["mailaddress"] = mailaddress
        body["dsm_lang"] = dsm_lang
        body["time_zone"] = time_zone
        body["sokind"] = "N"
        if locale:
            body["locale"] = locale
        return self._create(_hbs.HostBasedSecurity, **body)

    def change_hbs_menu(self, service_order_service, mailaddress, locale=None):
        """Change menu of Host-based Security.

        :param string service_order_service: Requested menu.
            Set "Managed Anti-Virus", "Managed Virtual Patch"
            or "Managed Host-based Security Package" to this field.
        :param string mailaddress: Contactable mail address.
        :param string locale: Messages are displayed in Japanese or English
                              depending on this value.
                              ja: Japanese, en: English. Default value is "en".
        :return: Host Based Security.
        :rtype: :class:`~ecl.security.v1.host_based_security.HostBasedSecurity`
        """
        body = {}
        body["tenant_id"] = self.session.get_project_id()
        body["service_order_service"] = service_order_service
        body["mailaddress"] = mailaddress
        body["sokind"] = "M1"
        if locale:
            body.update({"locale": locale})
        hbs = _hbs.HostBasedSecurity()
        return hbs.update(self.session, **body)

    def change_hbs_quantity(self, max_agent_value, mailaddress, locale=None):
        """Change maximum quantity of Agent usage.

        :param string max_agent_value: Set maximum quantity of Agenet usage.
        :param string mailaddress: Contactable mail address.
        :param string locale: Messages are displayed in Japanese or English
                              depending on this value.
                              ja: Japanese, en: English. Default value is "en".
        :return: Host Based Security.
        :rtype: :class:`~ecl.security.v1.host_based_security.HostBasedSecurity`
        """
        body = {}
        body["tenant_id"] = self.session.get_project_id()
        body["max_agent_value"] = max_agent_value
        body["mailaddress"] = mailaddress
        body["sokind"] = "M2"
        if locale:
            body.update({"locale": locale})
        hbs = _hbs.HostBasedSecurity()
        return hbs.update(self.session, **body)

    def cancel_hbs(self, mailaddress, locale=None):
        """Cancel the order of Host-based Security.

        :param string mailaddress: Contactable mail address.
        :param string locale: Messages are displayed in Japanese or English
                              depending on this value.
                              ja: Japanese, en: English. Default value is "en".
        :return: Host Based Security.
        :rtype: :class:`~ecl.security.v1.host_based_security.HostBasedSecurity`
        """
        body = {}
        body["tenant_id"] = self.session.get_project_id()
        body["mailaddress"] = mailaddress
        body["sokind"] = "C"
        if locale:
            body["locale"] = locale
        hbs = _hbs.HostBasedSecurity()
        return hbs.delete(self.session, body, locale=locale)



# -*- coding: utf-8 -*-

from ecl.tests.functional import base


class TestPort(base.BaseFunctionalTest):

    @classmethod
    def setUpClass(cls):
        super(TestPort, cls).setUpClass()
        cls.one_network = None
        sot = cls.conn.network.networks()
        if sot is not None and len(sot) > 0:
            cls.one_network = sot[0]
        cls.one_port = None
        ports = list(cls.conn.network.ports())
        for port in ports:
            if not port.managed_by_service and \
                    port.device_owner.startswith("compute"):
                cls.one_port = port
                break

    def test_find(self):
        sot = self.conn.network.find_port(self.one_port.id)
        self.assertEqual(self.one_port.id, sot.id)

    def test_get(self):
        sot = self.conn.network.get_port(self.one_port.id)
        self.assertEqual(self.one_port.id, sot.id)
        self.assertEqual(self.one_port.name, sot.name)
        self.assertEqual(self.one_port.network_id, sot.network_id)
        self.assertEqual(self.one_port.admin_state_up, sot.admin_state_up)

    def test_list(self):
        ids = [o.id for o in self.conn.network.ports()]
        self.assertIn(self.one_port.id, ids)

    def test_update(self):
        test_name = "test_port_name"
        test_description = "test_port_description"
        sot = self.conn.network.update_port(self.one_port.id,
                                            name=test_name,
                                            description=test_description)
        self.assertEqual(test_name, sot.name)
        self.assertEqual(test_description, sot.description)

    def test_create(self):
        test_name = "test_port"
        self.conn.network.create_port(name=test_name)

    def test_delete(self):
        self.conn.network.delete_port(self.one_port.id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import six
from ecl.tests.functional import base

class TestFirewallPlan(base.BaseFunctionalTest):

    def test_01_list_firewall_plans(self):
        plans = list(self.conn.network.firewall_plans(
            name="Brocade_5600_vRouter_3.5R6S3_4CPU-16GB-8IF",
            vendor="vyatta"
        ))
        print plans

    def test_02_show_firewall_plan(self):
        plan = self.conn.network.get_firewall_plan(
            "be90a148-99b9-4f69-b5cb-b03897eebd5d"
        )
        print plan
        self.assertIsInstance(plan.description, six.string_types)
        self.assertIsInstance(plan.id, six.string_types)
        self.assertIsInstance(plan.name, six.string_types)
        self.assertIsInstance(plan.vendor, six.string_types)
        self.assertIsInstance(plan.version, six.string_types)

    def test_02_find_firewall_plan(self):
        plan = self.conn.network.find_firewall_plan(
            "be90a148-99b9-4f69-b5cb-b03897eebd5d"
        )
        print plan
        self.assertIsInstance(plan.description, six.string_types)
        self.assertIsInstance(plan.id, six.string_types)
        self.assertIsInstance(plan.name, six.string_types)
        self.assertIsInstance(plan.vendor, six.string_types)
        self.assertIsInstance(plan.version, six.string_types)


# -*- coding: utf-8 -*-

import base
from ecl.network import network_service
from ecl import resource2
from ecl import utils


class FirewallInterface(base.NetworkBaseResource):
    resource_key = "firewall_interface"
    resources_key = "firewall_interfaces"
    service = network_service.NetworkService("v2.0")
    base_path = '/' + service.version + "/firewall_interfaces"

    _query_mapping = resource2.QueryParameters(
        "description", "firewall_id", "id",
        "ip_address", "name", "network_id",
        "slot_number", "status", "tenant_id",
        "virtual_ip_address", "sort_key", "sort_dir",
    )

    # Capabilities
    allow_get = True
    allow_list = True
    allow_update = True

    # Properties
    #: Description the Firewall Interface.
    description = resource2.Body("description")
    #: The ID of firewall this firewall_interface belongs to.
    firewall_id = resource2.Body("firewall_id")
    #: Unique ID of the Firewall Interface.
    id = resource2.Body("id")
    #: IP Address.
    ip_address = resource2.Body("ip_address")
    #: Name the Firewall Interface.
    name = resource2.Body("name")
    #: Network ID.
    network_id = resource2.Body("network_id")
    #: Slot Number.
    slot_number = resource2.Body("slot_number")
    #: Firewall interface status.
    status = resource2.Body("status")
    #: Tenant ID of the owner (UUID).
    tenant_id = resource2.Body("tenant_id")
    #: Virtual IP Address.
    virtual_ip_address = resource2.Body("virtual_ip_address")
    #: Properties used for virtual IP address.
    virtual_ip_properties = resource2.Body("virtual_ip_properties")

    #: VRID of Firewall virtual ip properties
    @property
    def vrid(self):
        virtual_ip_properties = self._body['virtual_ip_properties']
        if virtual_ip_properties is None:
            return
        else:
            return virtual_ip_properties['vrid']

    def update(self, session, firewall_id, firewall_interface_id, **attrs):
        """Update a firewall interface."""
        update_path = '/' + self.service.version
        uri = utils.urljoin(update_path, 'firewalls', firewall_id,
                            'firewall_interfaces', firewall_interface_id)
        resp = session.put(
            uri, endpoint_filter=self.service,
            json=attrs
        )
        self._translate_response(resp, has_body=True)
        return self


# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.dns import dns_service
from ecl import resource2
from ecl import exceptions


class RecordSet(resource2.Resource):
    resource_key = None
    resources_key = "recordsets"
    base_path = '/v2/zones/%(zone_id)s/recordsets'
    service = dns_service.DnsService()

    # Capabilities
    allow_create = True
    allow_delete = True
    allow_update = True
    allow_list = True
    allow_get = True

    _query_mapping = resource2.QueryParameters("limit", "marker")

    # Properties
    #: ID for the resource
    id = resource2.Body('id')
    #: DNS Name for the recordset.
    name = resource2.Body('name')
    #: Description for this recordset.
    description = resource2.Body('description')
    #: ID for the zone that contains this recordset.
    zone_id = resource2.URI('zone_id')
    #: Date / Time when resource was created.
    created_at = resource2.Body('created_at')
    #: Date / Time when resource last updated.
    updated_at = resource2.Body('updated_at')
    #: Version of the resource. This parameter is not currently supported. it always return 1.
    version = resource2.Body('version')
    #: They RRTYPE of the recordset.
    type = resource2.Body('type')
    #: status of the resource.
    status = resource2.Body('status')
    #: TTL (Time to Live) for the recordset. This parameter is not currently supported. it always return zero.
    ttl = resource2.Body('ttl')
    #: Links to the resource, and other related resources. When a response has been broken into pages,
    #: we will include a next link that should be followed to retrive all results.
    links = resource2.Body('links')
    #: Links to the resource, and other related resources. When a response has been broken into pages,
    #: we will include a next link that should be followed to retrive all results.
    records = resource2.Body('records')


    def _translate_recordsets(self, response, has_body=True):
        """
        In order to handle the response.
        Response example:
        {"recordsets":
        [
            {"id":"fcb86eb9-8f8d-4cfd-8309-9052236d75df",
                "zone_id":"d4f0ea0e-edb6-4bbb-aefd-2944457be234",
                "records":["203.0.143.22"],
                "ttl":3600,"name":"ns3.base.co.jp.",
                "description":null,"type":"A","version":1,
                "created_at":"","updated_at":null,
                "links":{"self":"https://dns-lab3ec-ecl.lab.api.ntt.com/v2/zones/d4f0ea0e-edb6-4bbb-aefd-2944457be234/recordsets/fcb86eb9-8f8d-4cfd-8309-9052236d75df"}},
            {"id":"b0590460-11b3-413d-ad95-5cd3f4b01c27",
                "zone_id":"d4f0ea0e-edb6-4bbb-aefd-2944457be234",
                "records":["203.0.143.23"],"ttl":3600,
                "name":"ns3.base.co.jp.",
                "description":null,"type":"A",
                "version":1,"created_at":"",
                "updated_at":null,
                "links":{"self":"https://dns-lab3ec-ecl.lab.api.ntt.com/v2/zones/d4f0ea0e-edb6-4bbb-aefd-2944457be234/recordsets/b0590460-11b3-413d-ad95-5cd3f4b01c27"}}
        ],
        "links":{"self":"https://dns-lab3ec-ecl.lab.api.ntt.com/v2/zones/d4f0ea0e-edb6-4bbb-aefd-2944457be234/recordsets"},
        "metadata":{"total_count":2}}
        """

        if has_body:
            body = response.json()
            body = body[self.resources_key]


            for data in body:
                value = self.existing(**data)
                yield value

    def create(self, session, prepend_key=True):
        """
        Recordset allow creating several records once.
        Thus return a list.
        """
        if not self.allow_create:
            raise exceptions.MethodNotSupported(self, "create")

        if self.put_create:
            request = self._prepare_request(requires_id=True,
                                            prepend_key=prepend_key)
            response = session.put(request.uri, endpoint_filter=self.service,
                                   json=request.body, headers=request.headers)
        else:
            request = self._prepare_request(requires_id=False,
                                            prepend_key=prepend_key)
            response = session.post(request.uri, endpoint_filter=self.service,
                                    json=request.body, headers=request.headers)

        return list(self._translate_recordsets(response, has_body=True))

    @classmethod
    def multi_delete(cls, session, zone_id, recordset_ids):
        """
        Delete multiple Recordsets
        """
        uri = cls.base_path % {"zone_id": zone_id}
        body = {"id": recordset_ids}
        session.delete(uri, endpoint_filter=cls.service, json=body)

    @classmethod
    def find(cls, session, name_or_id, zone_id, ignore_missing=False, **params):
        """Find a resource by its name or id.

        :param session: The session to use for making this request.
        :type session: :class:`~ecl.session.Session`
        :param name_or_id: This resource's identifier, if needed by
                           the request. The default is ``None``.
        :param zone_id: ID for the zone
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :param dict params: Any additional parameters to be passed into
                            underlying methods, such as to
                            :meth:`~ecl.resource2.Resource.existing`
                            in order to pass on URI parameters.

        :return: The :class:`Resource` object matching the given name or id
                 or None if nothing matches.
        :raises: :class:`ecl.exceptions.DuplicateResource` if more
                 than one resource is found for this request.
        :raises: :class:`ecl.exceptions.ResourceNotFound` if nothing
                 is found and ignore_missing is ``False``.
        """
        # Try to short-circuit by looking directly for a matching ID.

        data = list(cls.list(session, zone_id=zone_id, **params))

        result = cls._get_one_match(name_or_id, data)
        if result is not None:
            return result

        if ignore_missing:
            return None
        raise exceptions.ResourceNotFound(
            "No %s found for %s" % (cls.__name__, name_or_id))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import time

from ecl.tests.functional import base


class TestRecordSet(base.BaseFunctionalTest):

    @classmethod
    def test_01_create(cls):
        recordset1 = cls.conn.dns.create_recordset(
            zone_id="d4f0ea0e-edb6-4bbb-aefd-2944457be234",
            name="ns20.base.co.jp",
            type="A",
            ttl=3600,
            records=["203.0.143.27", "203.0.143.28"]
        )
        # print recordset1
        cls.recordset_id = recordset1[0].id
        cls.recordset2_id = recordset1[1].id
        time.sleep(10)

    def test_02_get(self):
        zone = self.conn.dns.get_recordset(zone_id="d4f0ea0e-edb6-4bbb-aefd-2944457be234",
                                           recordset_id=self.recordset_id)
        print zone

    def test_03_update(self):
        zone = self.conn.dns.update_recordset(zone_id="d4f0ea0e-edb6-4bbb-aefd-2944457be234", recordset=self.recordset_id, ttl=3000, description="for test use")
        time.sleep(10)
        # print zone

    def test_04_find(self):
        zone = self.conn.dns.find_recordset(zone_id="d4f0ea0e-edb6-4bbb-aefd-2944457be234", name_or_id="ns19.base.co.jp.")
        # self.assertEqual(zone.id, self.recordset_id)
        # print zone

    def test_05_delete(self):
        self.conn.dns.delete_recordset(zone_id="d4f0ea0e-edb6-4bbb-aefd-2944457be234", recordset=self.recordset_id)
        time.sleep(3)

    def test_06_list(self):
        zones = self.conn.dns.recordsets(zone_id="d4f0ea0e-edb6-4bbb-aefd-2944457be234")
        # print len(zones)

    def test_07_multi_delete(self):
        ids = [self.recordset2_id]
        self.conn.dns.delete_multiple_recordsets(
            "d4f0ea0e-edb6-4bbb-aefd-2944457be234",
            ids,
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from ecl.identity import identity_service
from ecl import service_filter


class TestValidVersion(testtools.TestCase):
    def test_constructor(self):
        sot = service_filter.ValidVersion('v1.0', 'v1')
        self.assertEqual('v1.0', sot.module)
        self.assertEqual('v1', sot.path)


class TestServiceFilter(testtools.TestCase):
    def test_init(self):
        sot = service_filter.ServiceFilter(
            'ServiceType', region='REGION1', service_name='ServiceName',
            version='1', api_version='1.23', requires_project_id=True)
        self.assertEqual('servicetype', sot.service_type)
        self.assertEqual('REGION1', sot.region)
        self.assertEqual('ServiceName', sot.service_name)
        self.assertEqual('1', sot.version)
        self.assertEqual('1.23', sot.api_version)
        self.assertTrue(sot.requires_project_id)

    def test_get_module(self):
        sot = identity_service.IdentityService()
        self.assertEqual('ecl.identity.v3', sot.get_module())
        self.assertEqual('identity', sot.get_service_module())



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl import resource
from ecl.telemetry import telemetry_service


class Resource(resource.Resource):
    """.. caution:: This API is a work in progress and is subject to change."""
    id_attribute = 'resource_id'
    base_path = '/resources'
    service = telemetry_service.TelemetryService()

    # Supported Operations
    allow_retrieve = True
    allow_list = True

    # Properties
    #: UTC date & time not later than the first sample known
    #: for this resource.
    first_sample_at = resource.prop('first_sample_timestamp')
    #: UTC date & time not earlier than the last sample known
    #: for this resource.
    last_sample_at = resource.prop('last_sample_timestamp')
    #: A list containing a self link and associated meter links
    links = resource.prop('links')
    #: Arbitrary metadata associated with the resource
    metadata = resource.prop('metadata')
    #: The ID of the owning project
    project_id = resource.prop('project_id')
    #: The ID for the resource
    resource_id = resource.prop('resource_id')
    #: The name of the source where the resource comes from
    source = resource.prop('source')
    #: The ID of the user who created the resource or updated it last
    user_id = resource.prop('user_id')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import unittest
import uuid

from ecl.tests.functional import base


@unittest.skip("bug/1524468")
@unittest.skipUnless(base.service_exists(service_type="metering"),
                     "Metering service does not exist")
class TestAlarmChange(base.BaseFunctionalTest):

    NAME = uuid.uuid4().hex
    alarm = None

    @classmethod
    def setUpClass(cls):
        super(TestAlarmChange, cls).setUpClass()
        meter = next(cls.conn.telemetry.meters())
        alarm = cls.conn.telemetry.create_alarm(
            name=cls.NAME,
            type='threshold',
            threshold_rule={
                'meter_name': meter.name,
                'threshold': 1.1,
            },
        )
        cls.alarm = alarm

    @classmethod
    def tearDownClass(cls):
        cls.conn.telemetry.delete_alarm(cls.alarm, ignore_missing=False)

    def test_list(self):
        change = next(self.conn.telemetry.alarm_changes(self.alarm))
        self.assertEqual(self.alarm.id, change.alarm_id)
        self.assertEqual('creation', change.type)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl import resource
from ecl.telemetry import telemetry_service


class Meter(resource.Resource):
    """.. caution:: This API is a work in progress and is subject to change."""
    id_attribute = 'meter_id'
    resource_key = 'meter'
    base_path = '/meters'
    service = telemetry_service.TelemetryService()

    # Supported Operations
    allow_list = True

    # Properties
    #: The ID of the meter
    meter_id = resource.prop('meter_id')
    #: The unique name for the meter
    name = resource.prop('name')
    #: The ID of the project that owns the resource
    project_id = resource.prop('project_id')
    #: The ID of the resource for which the measurements are taken
    resource_id = resource.prop('resource_id')
    #: The name of the source where the meter comes from
    source = resource.prop('source')
    #: The meter type
    type = resource.prop('type')
    #: The unit of measure
    unit = resource.prop('unit')
    #: The ID of the user who last triggered an update to the resource
    user_id = resource.prop('user_id')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import time

from ecl.tests.functional import base


class TestZone(base.BaseFunctionalTest):

    @classmethod
    def test_01_create(cls):
        zone = cls.conn.dns.create_zone(
            name="test.co.jp.")
        cls.zone_id = zone.id
        time.sleep(10)

    def test_02_get(self):
        zone = self.conn.dns.get_zone(self.zone_id)
        # print zone

    def test_03_update(self):
        zone = self.conn.dns.update_zone(self.zone_id, description="for test use")
        # print zone
        time.sleep(10)

    def test_04_find(self):
        zone = self.conn.dns.find_zone("test.co.jp.")
        self.assertEqual(zone.id, self.zone_id)

    def test_05_delete(self):
        self.conn.dns.delete_zone(self.zone_id)
        time.sleep(3)

    def test_06_list(self):
        zones = self.conn.dns.zones()
        # print zones

    def test_07_get_nameserver(self):
        nameserver = self.conn.dns.get_name_server(self.zone_id)
        #print nameserver



# -*- coding: utf-8 -*-

import base
from ecl.network import network_service
from ecl import resource2


class ColocationPhysicalLink(base.NetworkBaseResource):
    resource_key = 'colocation_physical_link'
    resources_key = 'colocation_physical_links'
    service = network_service.NetworkService("v2.0")
    base_path = '/' + service.version + '/colocation_physical_links'

    # capabilities
    allow_get = True
    allow_list = True

    # query mappings
    _query_mapping = resource2.QueryParameters('bandwidth',
                                               'colocation_space_id',
                                               'description',
                                               'id',
                                               'name',
                                               'status',
                                               'tenant_id',
                                               'type_a_physical_port_id',
                                               'type_a_rack_id',
                                               'type_b_physical_port_id',
                                               'type_b_rack_id',
                                               'colocation_physical_link_id',
                                               "sort_key", "sort_dir",)

    # properties
    #: The bandwidth of Colocation Physical Link.
    bandwidth = resource2.Body('bandwidth')
    #: Colocation Space ID.
    colocation_space_id = resource2.Body('colocation_space_id')
    #: Colocation Physical Link description.
    description = resource2.Body('description')
    #: Colocation Physical Link unique id.
    id = resource2.Body('id')
    #: Colocation Physical Link name.
    name = resource2.Body('name')
    #: Ranges of allowed VLAN tags.
    segmentation_ranges = resource2.Body('segmentation_ranges')
    #: The Colocation Physical Link status.
    status = resource2.Body('status')
    #: Colocation Physical Link tags.
    tags = resource2.Body('tags')
    #: The owner name of Colocation Physical Link.
    tenant_id = resource2.Body('tenant_id')
    #: Type A Physical Port ID.
    type_a_physical_port_id = resource2.Body('type_a_physical_port_id')
    #: Type A Rack ID.
    type_a_rack_id = resource2.Body('type_a_rack_id')
    #: Type B Physical Port ID.
    type_b_physical_port_id = resource2.Body('type_b_physical_port_id')
    #: Type B Rack ID.
    type_b_rack_id = resource2.Body('type_b_rack_id')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl import exceptions
from ecl.orchestration import orchestration_service
from ecl import resource2 as resource
from ecl import utils


class Stack(resource.Resource):
    name_attribute = 'stack_name'
    resource_key = 'stack'
    resources_key = 'stacks'
    base_path = '/stacks'
    service = orchestration_service.OrchestrationService()

    # capabilities
    allow_create = True
    allow_list = True
    allow_get = True
    allow_update = True
    allow_delete = True

    # Properties
    #: Placeholder for AWS compatible template listing capabilities
    #: required by the stack.
    capabilities = resource.Body('capabilities')
    #: Timestamp of the stack creation.
    created_at = resource.Body('creation_time')
    #: A text description of the stack.
    description = resource.Body('description')
    #: Whether the stack will support a rollback operation on stack
    #: create/update failures. *Type: bool*
    is_rollback_disabled = resource.Body('disable_rollback', type=bool)
    #: A list of dictionaries containing links relevant to the stack.
    links = resource.Body('links')
    #: Name of the stack.
    name = resource.Body('stack_name')
    #: Placeholder for future extensions where stack related events
    #: can be published.
    notification_topics = resource.Body('notification_topics')
    #: A list containing output keys and values from the stack, if any.
    outputs = resource.Body('outputs')
    #: The ID of the owner stack if any.
    owner_id = resource.Body('stack_owner')
    #: A dictionary containing the parameter names and values for the stack.
    parameters = resource.Body('parameters', type=dict)
    #: The ID of the parent stack if any
    parent_id = resource.Body('parent')
    #: A string representation of the stack status, e.g. ``CREATE_COMPLETE``.
    status = resource.Body('stack_status')
    #: A text explaining how the stack transits to its current status.
    status_reason = resource.Body('stack_status_reason')
    #: A dict containing the template use for stack creation.
    template = resource.Body('template', type=dict)
    #: Stack template description text. Currently contains the same text
    #: as that of the ``description`` property.
    template_description = resource.Body('template_description')
    #: A string containing the URL where a stack template can be found.
    template_url = resource.Body('template_url')
    #: Stack operation timeout in minutes.
    timeout_mins = resource.Body('timeout_mins')
    #: Timestamp of last update on the stack.
    updated_at = resource.Body('updated_time')
    #: The ID of the user project created for this stack.
    user_project_id = resource.Body('stack_user_project_id')
    #: A Environment information for stack
    environment = resource.Body('environment', type=dict)


    def stack_prepare_request(self, session, requires_id=True, prepend_key=False):
        """Prepare a request with auth header"""

        body = self._body.dirty
        if prepend_key and self.resource_key is not None:
            body = {self.resource_key: body}

        headers = self._header.dirty

        headers["X-Auth-User"] = session.auth._username
        headers["X-Auth-Key"] = session.auth._password

        uri = self.base_path % self._uri.attributes
        if requires_id:
            id = self._get_id(self)
            if id is None:
                raise exceptions.InvalidRequest(
                    "Request requires an ID but none was found")

            uri = utils.urljoin(uri, id)

        return resource._Request(uri, body, headers)

    def create(self, session):
        # This overrides the default behavior of resource creation because
        # heat doesn't accept resource_key in its request.
        # & Will use self.stack_prepare_request to get proper headers.
        if not self.allow_create:
            raise exceptions.MethodNotSupported(self, "create")

        if self.put_create:
            request = self.stack_prepare_request(session, requires_id=True,
                                            prepend_key=False)
            response = session.put(request.uri, endpoint_filter=self.service,
                                   json=request.body, headers=request.headers)
        else:
            request = self.stack_prepare_request(session, requires_id=False,
                                            prepend_key=False)
            response = session.post(request.uri, endpoint_filter=self.service,
                                    json=request.body, headers=request.headers)

        self._translate_response(response)
        return self

    def update(self, session):
        # This overrides the default behavior of resource creation because
        # heat doesn't accept resource_key in its request.
        return super(Stack, self).update(session, prepend_key=False,
                                         has_body=False)

    def _action(self, session, body):
        """Perform stack actions"""
        url = utils.urljoin(self.base_path, self._get_id(self), 'actions')
        resp = session.post(url, endpoint_filter=self.service, json=body)
        return resp.json()

    def check(self, session):
        return self._action(session, {'check': ''})

    def get(self, session, requires_id=True):
        stk = super(Stack, self).get(session, requires_id=requires_id)
        if stk and stk.status in ['DELETE_COMPLETE', 'ADOPT_COMPLETE']:
            raise exceptions.NotFoundException(
                "No stack found for %s" % stk.id)
        return stk

    def abandon(self, session):
        self.base_path = '/stacks/%s/%s/abandon' % (self.name, self.id)

        if not self.allow_delete:
            raise exceptions.MethodNotSupported(self, "delete")

        request = self._prepare_request(requires_id=False)

        response = session.delete(request.uri, endpoint_filter=self.service,
                                  headers={"Accept": ""})

        self._translate_response(response, has_body=False)
        return self


class StackPreview(Stack):
    base_path = '/stacks/preview'

    allow_create = True
    allow_list = False
    allow_get = False
    allow_update = False
    allow_delete = False



# -*- coding: utf-8 -*-

from ecl.provider_connectivity import provider_connectivity_service
from ecl.provider_connectivity.v1.base import ProviderConnectivityBaseResource
from ecl import resource2


class AWSConnection(ProviderConnectivityBaseResource):
    resources_key = "connections"
    resource_key = "connection"
    service = provider_connectivity_service.ProviderConnectivityService("v1.0")
    base_path = '/' + service.version + '/icc/aws/connections'

    # Capabilities
    allow_list = True
    allow_get = True
    allow_create = True
    allow_update = True
    allow_delete = True

    # _query_mapping = resource2.QueryParameters()
    # TBD

    # Properties
    #: It identifies connection resource uniquely.
    id = resource2.Body('id')
    #: Name of connection.
    name = resource2.Body('name')
    #: Description of connection.
    description = resource2.Body('description')
    #: Status of connection.
    status = resource2.Body('status')
    #: Tenant ID of the owner.
    tenant_id = resource2.Body('tenant_id')
    #: Bandwidth assigned with this connection.
    bandwidth = resource2.Body('bandwidth')
    #: QoS type assigned with this connection.
    qos_type = resource2.Body('qos_type')
    #: ID of exchange_points.
    exchange_point_id = resource2.Body('exchange_point_id')
    aws_account_id = resource2.Body('aws_account_id')
    connected_network_address = resource2.Body('connected_network_address')
    #: Network params between ECL2.0 and UNO.
    aws_network1 = resource2.Body('aws_network1')
    #: Network params between UNO and AWS.
    aws_network2 = resource2.Body('aws_network2')

    def approve(self, session, connection_id, action):
        """Approve/Disapprove connection between ECL2.0 and AWS."""

        uri = self.base_path + '/' + connection_id + '/action'
        resp = session.put(uri,
                           endpoint_filter=self.service,
                           json={"action": action})
        self._translate_response(resp, has_body=False)
        return self



# -*- coding: utf-8 -*-

import base

from ecl.virtual_network_appliance import virtual_network_appliance_service
from ecl import resource2
from ecl import utils


class VirtualNetworkAppliance(base.VirtualNetworkApplianceBaseResource):
    resources_key = "virtual_network_appliances"
    resource_key = "virtual_network_appliance"
    service = virtual_network_appliance_service.\
        VirtualNetworkApplianceService("v1.0")
    base_path = '/' + service.version + '/virtual_network_appliances'

    patch_update = True

    # Capabilities
    allow_list = True
    allow_get = True
    allow_create = True
    allow_update = True
    allow_delete = True

    # _query_mapping = resource2.QueryParameters()
    # TBD

    # Properties
    #: It identifies connection resource uniquely.
    id = resource2.Body('id')
    #: Name of virtual network appliance
    name = resource2.Body('name')
    #: Description of virtual network appliance
    description = resource2.Body('description')
    #: Tags of virtual network appliance
    tags = resource2.Body('tags')
    #: Appliance type of virtual network appliance
    appliance_type = resource2.Body('appliance_type')
    #: Zone of virtual network appliance
    availability_zone = resource2.Body('availability_zone')
    #: Default gateway of virtual network appliance
    default_gateway = resource2.Body('default_gateway')
    #: OS monitoring status of virtual network appliance
    os_monitoring_status = resource2.Body('os_monitoring_status')
    #: OS login status of virtual network appliance
    os_login_status = resource2.Body('os_login_status')
    #: VM status of virtual network appliance
    vm_status = resource2.Body('vm_status')
    #: Operation status of virtual network appliance
    operation_status = resource2.Body('operation_status')
    #: Plan ID of virtual network appliance
    virtual_network_appliance_plan_id = \
        resource2.Body('virtual_network_appliance_plan_id')
    #: Tenant ID of virtual network appliance.
    tenant_id = resource2.Body('tenant_id')
    #: Interface definition of virtual network appliance.
    interfaces = resource2.Body('interfaces')
    #: User name of virtual network appliance.
    username = resource2.Body('username')
    #: Password of virtual network appliance.
    password = resource2.Body('password')
    #: Password(after reset) of virtual network appliance.
    new_password = resource2.Body('new_password')

    def update(self, session, prepend_key=True, has_body=True):
        """Update the remote resource based on this virtual network appliance.

        Reason why override this method manually is,
        virtual network appliance API only allows PATCH method as
        way of updating.

        :param session: The session to use for making this request.
        :type session: :class:`~ecl.session.Session`
        :param prepend_key: A boolean indicating whether the resource_key
            should be prepended in a resource update request.
            Default to True.

        :return: This :class:`Resource` instance.
        :raises: :exc:`~ecl.exceptions.MethodNotSupported` if
                 :data:`Resource.allow_update` is not set to ``True``.
        """
        if not any([self._body.dirty, self._header.dirty]):
            return self

        request = self._prepare_request(prepend_key=prepend_key)

        if 'id' in request.body[self.resource_key]:
            del request.body[self.resource_key]['id']

        response = session.patch(request.uri, endpoint_filter=self.service,
                                 json=request.body,
                                 headers=request.headers)

        self._translate_response(response, has_body=has_body)
        return self

    def _action(self, session, body, postfix='action'):
        """Preform virtual network appliance actions given the message body"""
        url = utils.urljoin(VirtualNetworkAppliance.base_path,
                            self.id, postfix)
        headers = {'Accept': ''}
        return session.post(
            url, endpoint_filter=self.service, json=body, headers=headers)

    def start(self, session):
        """Start virtual network appliance"""
        body = {"os-start": None}
        return self._action(session, body)

    def stop(self, session):
        """Stop virtual network appliance"""
        body = {"os-stop": None}
        return self._action(session, body)

    def restart(self, session):
        """Restart virtual network appliance"""
        body = {'os-restart': None}
        self._action(session, body)

    def reset_password(self, session):
        """Reset virtual network appliance password."""
        body = {}
        resp = self._action(session, body, postfix='reset-password')
        self._translate_response(resp, has_body=True)
        return self

    def get_console(self, session, vnc_type):
        """Get console of virtual network appliance"""
        action = {'type': vnc_type}
        body = {'os-getVNCConsole': action}
        resp = self._action(session, body, postfix='remote-console')
        if resp:
            console_dict = resp.json()
            if console_dict:
                return console_dict.get("console")



# -*- coding: utf-8 -*-

from ecl.provider_connectivity import provider_connectivity_service
from ecl.provider_connectivity.v2.base import ProviderConnectivityBaseResource
from ecl import resource2


class TenantConnectionRequest(ProviderConnectivityBaseResource):
    resources_key = "tenant_connection_requests"
    resource_key = "tenant_connection_request"
    service = provider_connectivity_service.ProviderConnectivityService("v2.0")
    base_path = '/' + service.version + '/tenant_connection_requests'

    # Capabilities
    allow_list = True
    allow_get = True
    allow_create = True
    allow_update = True
    allow_delete = True

    # query mappings
    _query_mapping = resource2.QueryParameters(
        'tenant_connection_request_id',
        'status',
        'name',
        'tenant_id',
        'name_other',
        'tenant_id_other',
        'network_id',
        'approval_request_id',
        'keystone_user_id',
    )

    #: tenant_connection_request unique ID.
    tenant_connection_request_id = resource2.Body(
        'tenant_connection_request_id', alternate_id=True)

    id = resource2.Body("id")

    #: Name of tenant_connection_request.
    name = resource2.Body('name')

    #: Description
    description = resource2.Body('description')

    #: Status of tenant_connection_request.
    status = resource2.Body('status')

    #: Tags
    tags = resource2.Body('tags')

    #: Keystone User ID who can access to the owner tenant of
    # tenant_connection_request.
    keystone_user_id = resource2.Body('keystone_user_id')

    #: Tenant ID of the owner.
    tenant_id = resource2.Body('tenant_id')

    #: Name for the owner of network
    name_other = resource2.Body('name_other')

    #: Description for the owner of the network
    description_other = resource2.Body('description_other')

    #: Tags for the owner of the network
    tags_other = resource2.Body('tags_other')

    #: The owner tenant of the network
    tenant_id_other = resource2.Body('tenant_id_other')

    #: Network unique ID
    network_id = resource2.Body('network_id')

    #: Approval unique ID
    approval_request_id = resource2.Body('approval_request_id')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
Load various modules for authorization and eventually services.
"""
from stevedore import extension


def load_service_plugins(namespace):
    service_plugins = extension.ExtensionManager(
        namespace=namespace,
        invoke_on_load=True,
    )
    services = {}
    for service in service_plugins:
        service = service.obj
        service.interface = None
        services[service.service_type] = service
    return services



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl import proxy2
from ecl.dns.v2 import zone as _zone
from ecl.dns.v2 import name_server
from ecl.dns.v2 import recordset as _recordset


class Proxy(proxy2.BaseProxy):

    def zones(self):
        """
        List the zones.
        :return: A list of zone object
        """
        return list(self._list(_zone.Zone))

    def get_zone(self, zone):
        """
        Show a Zone.
        :param: zone: The value can be the ID of a zone or a
                       :class:`~ecl.dns.v2.zone.Zone` instance.
        :return: One :class:`~ecl.dns.v2.zone.Zone` or
                     :class:`~ecl.exceptions.ResourceNotFound`when no
                     resource can be found.
        """
        return self._get(_zone.Zone, zone)

    def find_zone(self, name_or_id, ignore_missing=False):
        """
        Find a zone by its name or ID
        :param name_or_id: The name or ID of a zone
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :return: One :class:`~ecl.dns.v2.zone.Zone` or
                     :class:`~ecl.exceptions.ResourceNotFound`when no
                     resource can be found.
        """
        return self._find(_zone.Zone, name_or_id, ignore_missing=ignore_missing)

    def create_zone(self, name=None, ttl=None, type=None,
                    description=None, email=None, masters=None):
        """
        Create a zone.
        :param name: DNS Name for the zone. Required.
        :param ttl: TTL (Time to Live) for the zone. This parameter is not currently supported.
        :param type: Type of zone. PRIMARY is controlled by ECL2.0 DNS, SECONDARY zones are slaved from another DNS Server. Defaults to PRIMARY. This parameter is not currently supported.
        :param description: Description for this zone
        :param email: e-mail for the zone. Used in SOA records for the zone. This parameter is not currently supported.
        :param masters: For secondary zones. The servers to slave from to get DNS information. This parameter is not currently supported.
        :return: :class:`~ecl.dns.v2.zone.Zone`
        """
        attr = {"name": name}
        if ttl is not None:
            attr["ttl"] = ttl
        if type is not None:
            attr["type"] = type
        if description is not None:
            attr["description"] = description
        if email is not None:
            attr["email"] = email
        if masters is not None:
            attr["masters"] = masters
        return self._create(_zone.Zone, **attr)

    def update_zone(self, zone, ttl=None,
                    description=None, email=None, masters=None):
        """
        Update the attribute(s) for an existing zone.
        :param zone: ID for the zone or zone instance to update.
        :param ttl: TTL (Time to Live) for the zone. This parameter is not currently supported.
        :param description: Description for this zone
        :param email: e-mail for the zone. Used in SOA records for the zone. This parameter is not currently supported.
        :param masters: For secondary zones. The servers to slave from to get DNS information. This parameter is not currently supported.
        :return: :class:`~ecl.dns.v2.zone.Zone`
        """
        attr = {}
        if ttl is not None:
            attr["ttl"] = ttl
        if description is not None:
            attr["description"] = description
        if email is not None:
            attr["email"] = email
        if masters is not None:
            attr["masters"] = masters

        if not isinstance(zone, _zone.Zone):
            zone = self._get_resource(_zone.Zone, zone)
            zone._body.clean()

        return self._update(_zone.Zone, zone, **attr)

    def delete_zone(self, zone_id, ignore_missing=False):
        """
        Delete a zone.
        :param zone_id: ID for the zone
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the server does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent zone
        :return: None
        """
        return self._delete(_zone.Zone, zone_id, ignore_missing=False)

    def get_name_server(self, zone_id):
        """
        Show the nameservers for a zone.
        :param zone_id: ID for the zone
        :return: :class:`~ecl.dns.v2.name_server.NameServer`
        """
        return list(self._list(name_server.NameServer, zone_id=zone_id))

    def recordsets(self, zone_id, limit=None, marker=None):
        """
        This lists all recordsets in a zone.
        :param zone_id: ID for the zone
        :param limit: Requests a page size of items(1-500). Returns a number of items up to a limit value. Use the limit parameter to make an initial limited request and use the ID of the last-seen item from the response as the marker parameter value in a subsequent limited request.
        :param marker: The ID of the last-seen item. Use the limit parameter to make an initial limited request and use the ID of the last-seen item from the response as the marker parameter value in a subsequent limited request.
        :return: One list of :class:`~ecl.dns.v2.recordsets.Recordsets`
        """
        attrs = {}
        attrs["zone_id"] = zone_id
        if limit is not None:
            attrs["limit"] = limit
        if marker is not None:
            attrs["marker"] = marker

        return list(self._list(_recordset.RecordSet, **attrs))

    def get_recordset(self, zone_id, recordset_id):
        """
        Show a single recordset.
        :param zone_id: ID for the zone the recordset belonging to
        :param recordset_id: ID for the recordset
        :return: One :class:`~ecl.dns.v2.recordsets.Recordsets` or
                     :class:`~ecl.exceptions.ResourceNotFound`when no
                     resource can be found.
        """
        # TODO: If zone instance is used...
        return self._get(_recordset.RecordSet, recordset_id, zone_id=zone_id)

    def find_recordset(self, zone_id, name_or_id, ignore_missing=False):
        """
        Find a recordset by its name or ID.
        :param name_or_id: Name or ID for this recordset
        :return: One :class:`~ecl.dns.v2.recordsets.Recordsets` or
                     :class:`~ecl.exceptions.ResourceNotFound`when no
                     resource can be found.
        """
        return self._find(_recordset.RecordSet, name_or_id, zone_id=zone_id, ignore_missing=ignore_missing)

    def create_recordset(self, zone_id, name=None, description=None,
                         type=None, ttl=None, records=None):
        """
        Create a recordset in a zone.
        :param zone_id: ID for the zone.
        :param name: DNS Name for the recordset.
        :param description: Description for this recordset.
        :param type: RRTYPE of the recordset. Valid Values: A | AAAA | MX | CNAME | SRV | SPF | TXT | PTR | NS
        :param ttl: TTL (Time to Live) for the recordset.
        :param recodrs: A list of data for this recordset.
                        Each item will be a separate record in ECL2.0 DNS.
                        These items should conform to the DNS spec for the record type
                        - e.g. A records must be IPv4 addresses, CNAME records must be a hostname.
        :return: :class:`~ecl.dns.v2.recordsets.Recordsets`
        """
        attr = {"name": name}
        if ttl is not None:
            attr["ttl"] = ttl
        if type is not None:
            attr["type"] = type
        if description is not None:
            attr["description"] = description
        if records is not None:
            attr["records"] = records

        return self._create(_recordset.RecordSet, zone_id=zone_id, **attr)

    def update_recordset(self, zone_id, recordset, ttl=None, description=None,
                         name=None, records=None):
        """
        Update a recordset.
        :param recordset: ID for the recordset.
        :param zone_id: ID for the zone.
        :param ttl: TTL (Time to Live) for the recordset.
        :param description: Description for this recordset.
        :param name: DNS Name for the recordset.
        :param record: A list of data for this recordset.
                        Each item will be a separate record in ECL2.0 DNS.
                        These items should conform to the DNS spec for the record type
                        - e.g. A records must be IPv4 addresses, CNAME records must be a hostname.
        :return: :class:`~ecl.dns.v2.recordsets.Recordsets`
        """
        attr = {}
        if ttl is not None:
            attr["ttl"] = ttl
        if description is not None:
            attr["description"] = description
        if records is not None:
            attr["records"] = records
        if name is not None:
            attr["name"] = name

        if not isinstance(recordset, _recordset.RecordSet):
            recordset = self._get_resource(_recordset.RecordSet, recordset)
            recordset._body.clean()

        return self._update(_recordset.RecordSet,
                            recordset, zone_id=zone_id, **attr)

    def delete_recordset(self, zone_id, recordset):
        """
        Delete a recordset.
        :param zone_id: ID for the recordset.
        :param recordset: ID for the zone.
        :return: None
        """
        return self._delete(_recordset.RecordSet, recordset, zone_id=zone_id)

    def delete_multiple_recordsets(self, zone_id, recordset_ids):
        """
        Delete multiple Recordsets.
        :param recordset_ids: A list of IDs for the recordsets.
        :param zone_id: ID for the zone
        :return: None
        """
        return _recordset.RecordSet.multi_delete(
                self.session, zone_id=zone_id,
                recordset_ids=recordset_ids
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import unittest

from ecl import exceptions
from ecl.orchestration.v1 import stack
from ecl.tests.functional import base
from ecl.tests.functional.network.v2 import test_network


@unittest.skip("bug/1525005")
class TestStack(base.BaseFunctionalTest):

    NAME = 'test_stack'
    stack = None
    network = None
    subnet = None
    cidr = '10.99.99.0/16'

    @classmethod
    def setUpClass(cls):
        super(TestStack, cls).setUpClass()
        if cls.conn.compute.find_keypair(cls.NAME) is None:
            cls.conn.compute.create_keypair(name=cls.NAME)
        image = next(cls.conn.image.images())
        tname = "ecl/tests/functional/orchestration/v1/hello_world.yaml"
        with open(tname) as f:
            template = f.read()
        cls.network, cls.subnet = test_network.create_network(cls.conn,
                                                              cls.NAME,
                                                              cls.cidr)
        parameters = {
            'image': image.id,
            'key_name': cls.NAME,
            'network': cls.network.id,
        }
        sot = cls.conn.orchestration.create_stack(
            name=cls.NAME,
            parameters=parameters,
            template=template,
        )
        assert isinstance(sot, stack.Stack)
        cls.assertIs(True, (sot.id is not None))
        cls.stack = sot
        cls.assertIs(cls.NAME, sot.name)
        cls.conn.orchestration.wait_for_status(
            sot, status='CREATE_COMPLETE', failures=['CREATE_FAILED'])

    @classmethod
    def tearDownClass(cls):
        super(TestStack, cls).tearDownClass()
        cls.conn.orchestration.delete_stack(cls.stack, ignore_missing=True)
        cls.conn.compute.delete_keypair(cls.NAME)
        # Need to wait for the stack to go away before network delete
        try:
            cls.conn.orchestration.wait_for_status(
                cls.stack, 'DELETE_COMPLETE')
        except exceptions.NotFoundException:
            pass
        cls.linger_for_delete()
        test_network.delete_network(cls.conn, cls.network, cls.subnet)

    def test_list(self):
        names = [o.name for o in self.conn.orchestration.stacks()]
        self.assertIn(self.NAME, names)



# -*- coding: utf-8 -*-


from ecl.connectivity import connectivity_service
from ecl import resource2


class MCIC(resource2.Resource):
    resource_key = None
    resources_key = None
    base_path = '/mCICs'
    service = connectivity_service.ConnectivityService()

    # capabilities
    allow_get = True
    allow_list = True

    _query_mapping = resource2.QueryParameters("tenant_id", "sort_key",
                                               "sort_dir")

    mcic_id = resource2.Body('mcic_id', alternate_id=True)
    mcic_name = resource2.Body('mcic_name')
    mcic_status = resource2.Body('mcic_status')
    tenant_id = resource2.Body('tenant_id')
    tenant_name = resource2.Body('tenant_name')
    service_type = resource2.Body('service_type')
    ngc = resource2.Body('ngc', dict)
    colo = resource2.Body('colo', dict)
    timezone = resource2.Body('timezone')


class MEIC(MCIC):
    mcic_id = resource2.Body('mcic_id', alternate_id=True)
    tenant_name = resource2.Body('tenant_name')
    service_id = resource2.Body('service_id')
    colo = None
    ec = resource2.Body('ec', dict)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.identity import identity_service
from ecl import resource


class Domain(resource.Resource):
    resource_key = 'domain'
    resources_key = 'domains'
    base_path = '/domains'
    service = identity_service.IdentityService()

    # capabilities
    allow_create = True
    allow_retrieve = True
    allow_update = True
    allow_delete = True
    allow_list = True
    patch_update = True

    # Properties
    #: The description of this domain. *Type: string*
    description = resource.prop('description')
    #: Setting this attribute to ``False`` prevents users from authorizing
    #: against this domain or any projects owned by this domain, and prevents
    #: users owned by this domain from authenticating or receiving any other
    #: authorization. Additionally, all pre-existing tokens applicable
    #: to the above entities are immediately invalidated.
    #: Re-enabling a domain does not re-enable pre-existing tokens.
    #: *Type: bool*
    is_enabled = resource.prop('enabled', type=bool)
    #: The globally unique name of this domain. *Type: string*
    name = resource.prop('name')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os

import fixtures
import mock
import os_client_config

from ecl import connection
from ecl import profile
from ecl.tests.unit import base


CONFIG_AUTH_URL = "http://127.0.0.1:5000/v2.0"
CONFIG_USERNAME = "BozoTheClown"
CONFIG_PASSWORD = "TopSecret"
CONFIG_PROJECT = "TheGrandPrizeGame"
CONFIG_CACERT = "TrustMe"

CLOUD_CONFIG = """
clouds:
  sample:
    region_name: RegionOne
    auth:
      auth_url: {auth_url}
      username: {username}
      password: {password}
      project_name: {project}
  insecure:
    auth:
      auth_url: {auth_url}
      username: {username}
      password: {password}
      project_name: {project}
    cacert: {cacert}
    insecure: True
  cacert:
    auth:
      auth_url: {auth_url}
      username: {username}
      password: {password}
      project_name: {project}
    cacert: {cacert}
    insecure: False
""".format(auth_url=CONFIG_AUTH_URL, username=CONFIG_USERNAME,
           password=CONFIG_PASSWORD, project=CONFIG_PROJECT,
           cacert=CONFIG_CACERT)


class TestConnection(base.TestCase):
    @mock.patch("ecl.session.Session")
    def test_other_parameters(self, mock_session_init):
        mock_session_init.return_value = mock_session_init
        mock_profile = mock.Mock()
        mock_profile.get_services = mock.Mock(return_value=[])
        conn = connection.Connection(profile=mock_profile, authenticator='2',
                                     verify=True, cert='cert', user_agent='1')
        args = {'auth': '2', 'user_agent': '1', 'verify': True, 'cert': 'cert'}
        mock_session_init.assert_called_with(mock_profile, **args)
        self.assertEqual(mock_session_init, conn.session)

    @mock.patch("keystoneauth1.loading.base.get_plugin_loader")
    def test_create_authenticator(self, mock_get_plugin):
        mock_plugin = mock.Mock()
        mock_loader = mock.Mock()
        mock_options = [
            mock.Mock(dest="auth_url"),
            mock.Mock(dest="password"),
            mock.Mock(dest="username"),
        ]
        mock_loader.get_options = mock.Mock(return_value=mock_options)
        mock_loader.load_from_options = mock.Mock(return_value=mock_plugin)
        mock_get_plugin.return_value = mock_loader
        auth_args = {
            'auth_url': '0',
            'username': '1',
            'password': '2',
        }
        conn = connection.Connection(auth_plugin='v2password', **auth_args)
        mock_get_plugin.assert_called_with('v2password')
        mock_loader.load_from_options.assert_called_with(**auth_args)
        self.assertEqual(mock_plugin, conn.authenticator)

    @mock.patch("keystoneauth1.loading.base.get_plugin_loader")
    def test_default_plugin(self, mock_get_plugin):
        connection.Connection()
        self.assertTrue(mock_get_plugin.called)
        self.assertEqual(mock_get_plugin.call_args, mock.call("password"))

    @mock.patch("keystoneauth1.loading.base.get_plugin_loader")
    def test_pass_authenticator(self, mock_get_plugin):
        mock_plugin = mock.Mock()
        mock_get_plugin.return_value = None
        conn = connection.Connection(authenticator=mock_plugin)
        self.assertFalse(mock_get_plugin.called)
        self.assertEqual(mock_plugin, conn.authenticator)

    def test_create_session(self):
        auth = mock.Mock()
        prof = profile.Profile()
        conn = connection.Connection(authenticator=auth, profile=prof)
        self.assertEqual(auth, conn.authenticator)
        self.assertEqual(prof, conn.profile)
        self.assertEqual('ecl.baremetal.v2._proxy',
                         conn.baremetal.__class__.__module__)
        self.assertEqual('ecl.compute.v2._proxy',
                         conn.compute.__class__.__module__)
        self.assertEqual('ecl.connectivity.v1._proxy',
                         conn.connectivity.__class__.__module__)
        self.assertEqual('ecl.dedicated_hypervisor.v1._proxy',
                         conn.dedicated_hypervisor.__class__.__module__)
        self.assertEqual('ecl.identity.v3._proxy',
                         conn.identity.__class__.__module__)
        self.assertEqual('ecl.image.v2._proxy',
                         conn.image.__class__.__module__)
        self.assertEqual('ecl.network.v2._proxy',
                         conn.network.__class__.__module__)
        self.assertEqual('ecl.rca.v1._proxy',
                         conn.rca.__class__.__module__)
        self.assertEqual('ecl.orchestration.v1._proxy',
                         conn.orchestration.__class__.__module__)
        self.assertEqual('ecl.telemetry.v2._proxy',
                         conn.telemetry.__class__.__module__)
        self.assertEqual('ecl.sss.v1._proxy',
                         conn.sss.__class__.__module__)

    def _prepare_test_config(self):
        # Create a temporary directory where our test config will live
        # and insert it into the search path via OS_CLIENT_CONFIG_FILE.
        config_dir = self.useFixture(fixtures.TempDir()).path
        config_path = os.path.join(config_dir, "clouds.yaml")

        with open(config_path, "w") as conf:
            conf.write(CLOUD_CONFIG)

        self.useFixture(fixtures.EnvironmentVariable(
            "OS_CLIENT_CONFIG_FILE", config_path))

    def test_from_config_given_data(self):
        self._prepare_test_config()

        data = os_client_config.OpenStackConfig().get_one_cloud("sample")

        sot = connection.from_config(cloud_config=data)

        self.assertEqual(CONFIG_USERNAME,
                         sot.authenticator._username)
        self.assertEqual(CONFIG_PASSWORD,
                         sot.authenticator._password)
        self.assertEqual(CONFIG_AUTH_URL,
                         sot.authenticator.auth_url)
        self.assertEqual(CONFIG_PROJECT,
                         sot.authenticator._project_name)

    def test_from_config_given_name(self):
        self._prepare_test_config()

        sot = connection.from_config(cloud_name="sample")

        self.assertEqual(CONFIG_USERNAME,
                         sot.authenticator._username)
        self.assertEqual(CONFIG_PASSWORD,
                         sot.authenticator._password)
        self.assertEqual(CONFIG_AUTH_URL,
                         sot.authenticator.auth_url)
        self.assertEqual(CONFIG_PROJECT,
                         sot.authenticator._project_name)

    def test_from_config_given_options(self):
        self._prepare_test_config()

        version = "100"

        class Opts(object):
            compute_api_version = version

        sot = connection.from_config(cloud_name="sample", options=Opts)

        pref = sot.session.profile.get_filter("compute")

        # NOTE: Along the way, the `v` prefix gets added so we can build
        # up URLs with it.
        self.assertEqual("v" + version, pref.version)

    def test_from_config_verify(self):
        self._prepare_test_config()

        sot = connection.from_config(cloud_name="insecure")
        self.assertFalse(sot.session.verify)

        sot = connection.from_config(cloud_name="cacert")
        self.assertEqual(CONFIG_CACERT, sot.session.verify)

    def test_authorize_works(self):
        fake_session = mock.Mock()
        fake_headers = {'X-Auth-Token': 'FAKE_TOKEN'}
        fake_session.get_auth_headers.return_value = fake_headers

        sot = connection.Connection(session=fake_session,
                                    authenticator=mock.Mock())
        res = sot.authorize()
        self.assertEqual('FAKE_TOKEN', res)

    def test_authorize_silent_failure(self):
        fake_session = mock.Mock()
        fake_session.get_auth_headers.return_value = None

        sot = connection.Connection(session=fake_session,
                                    authenticator=mock.Mock())
        res = sot.authorize()
        self.assertIsNone(res)
